@@ -60,7 +60,7 @@ Example set of cookies:
 #include <ctype.h>
 
 #include "cookie.h"
-#include "../include/setup.h"
+#include "setup.h"
 #include "getdate.h"
 
 /****************************************************************************
@@ -72,13 +72,14 @@ Example set of cookies:
  ***************************************************************************/
 
 struct Cookie *cookie_add(struct CookieInfo *c,
-                          char *lineptr) /* first non-space after the
-                                            Set-Cookie: prefix */
+                          bool httpheader, /* TRUE if HTTP header-style line */
+                          char *lineptr) /* first non-space of the line */
 {
   struct Cookie *clist;
   char what[MAX_COOKIE_LINE];
   char name[MAX_NAME];
   char *ptr;
+  char *semiptr;
   struct Cookie *co;
   time_t now = time(NULL);
   bool replace_old = FALSE;
@@ -91,49 +92,147 @@ struct Cookie *cookie_add(struct CookieInfo *c,
   /* clear the whole struct first */
   memset(co, 0, sizeof(struct Cookie));
 	    
-  ptr=strtok(lineptr, ";"); /* first tokenize it on the semicolon */
-  while(ptr) {
-    /* we have a <what>=<this> pair or a 'secure' word here */
-    if(strchr(ptr, '=')) {
-      if(2 == sscanf(ptr, "%" MAX_NAME_TXT "[^=]=%"
-                     MAX_COOKIE_LINE_TXT "[^\r\n]",
-                     name, what)) {
-        /* this is a legal <what>=<this> pair */
-        if(strequal("path", name)) {
-          co->path=strdup(what);
+  if(httpheader) {
+    /* This line was read off a HTTP-header */
+
+    semiptr=strchr(lineptr, ';'); /* first, find a semicolon */
+    ptr = lineptr;
+    while(semiptr) {
+      *semiptr='\0'; /* zero terminate for a while */
+      /* we have a <what>=<this> pair or a 'secure' word here */
+      if(strchr(ptr, '=')) {
+        name[0]=what[0]=0; /* init the buffers */
+        if(1 <= sscanf(ptr, "%" MAX_NAME_TXT "[^=]=%"
+                       MAX_COOKIE_LINE_TXT "[^\r\n]",
+                       name, what)) {
+          /* this is a legal <what>=<this> pair */
+          if(strequal("path", name)) {
+            co->path=strdup(what);
+          }
+          else if(strequal("domain", name)) {
+            co->domain=strdup(what);
+          }
+          else if(strequal("version", name)) {
+            co->version=strdup(what);
+          }
+          else if(strequal("max-age", name)) {
+            /* Defined in RFC2109:
+
+               Optional.  The Max-Age attribute defines the lifetime of the
+               cookie, in seconds.  The delta-seconds value is a decimal non-
+               negative integer.  After delta-seconds seconds elapse, the
+               client should discard the cookie.  A value of zero means the
+               cookie should be discarded immediately.
+
+             */
+            co->maxage = strdup(what);
+            co->expires =
+              atoi((*co->maxage=='\"')?&co->maxage[1]:&co->maxage[0]);
+          }
+          else if(strequal("expires", name)) {
+            co->expirestr=strdup(what);
+            co->expires = get_date(what, &now);
+          }
+          else if(!co->name) {
+            co->name = strdup(name);
+            co->value = strdup(what);
+          }
+          else
+            ;/* this is the second (or more) name we don't know
+                about! */
         }
-        else if(strequal("domain", name)) {
-          co->domain=strdup(what);
+        else {
+          /* this is an "illegal" <what>=<this> pair */
         }
-        else if(strequal("expires", name)) {
-          co->expirestr=strdup(what);
-          co->expires = get_date(what, &now);
-        }
-        else if(!co->name) {
-          co->name = strdup(name);
-          co->value = strdup(what);
-        }
-        else
-          ;/* this is the second (or more) name we don't know
-              about! */
       }
       else {
-        /* this is an "illegal" <what>=<this> pair */
+        if(sscanf(ptr, "%" MAX_COOKIE_LINE_TXT "[^\r\n]",
+                  what)) {
+          if(strequal("secure", what))
+            co->secure = TRUE;
+          else
+            ; /* unsupported keyword without assign! */
+        }
       }
+      *semiptr=';'; /* put the semicolon back */
+      ptr=semiptr+1;
+      while(ptr && *ptr && isspace((int)*ptr))
+        ptr++;
+      semiptr=strchr(ptr, ';'); /* now, find the next semicolon */
     }
-    else {
-      if(sscanf(ptr, "%" MAX_COOKIE_LINE_TXT "[^\r\n]",
-                what)) {
-        if(strequal("secure", what))
-          co->secure = TRUE;
-        else
-          ; /* unsupported keyword without assign! */
-      }
-    }
-    ptr=strtok(NULL, ";");
-    while(ptr && *ptr && isspace(*ptr))
-      ptr++;
   }
+  else {
+    /* This line is NOT a HTTP header style line, we do offer support for
+       reading the odd netscape cookies-file format here */
+    char *firstptr;
+    int fields;
+
+    if(lineptr[0]=='#') {
+      /* don't even try the comments */
+      free(co);
+      return NULL;
+    }
+    /* strip off the possible end-of-line characters */
+    if(ptr=strchr(lineptr, '\r'))
+      *ptr=0; /* clear it */
+    if(ptr=strchr(lineptr, '\n'))
+      *ptr=0; /* clear it */
+
+    firstptr=strtok(lineptr, "\t"); /* first tokenize it on the TAB */
+
+    /* Here's a quick check to eliminate normal HTTP-headers from this */
+    if(!firstptr || strchr(firstptr, ':')) {
+      free(co);
+      return NULL;
+    }
+
+    /* Now loop through the fields and init the struct we already have
+       allocated */
+    for(ptr=firstptr, fields=0; ptr; ptr=strtok(NULL, "\t"), fields++) {
+      switch(fields) {
+      case 0:
+        co->domain = strdup(ptr);
+        break;
+      case 1:
+        /* what _is_ this field for? */
+        break;
+      case 2:
+        co->path = strdup(ptr);
+        break;
+      case 3:
+        co->secure = strequal(ptr, "TRUE");
+        break;
+      case 4:
+        co->expires = atoi(ptr);
+        break;
+      case 5:
+        co->name = strdup(ptr);
+        break;
+      case 6:
+        co->value = strdup(ptr);
+        break;
+      }
+    }
+
+    if(7 != fields) {
+      /* we did not find the sufficient number of fields to recognize this
+         as a valid line, abort and go home */
+
+      if(co->domain)
+        free(co->domain);
+      if(co->path)
+        free(co->path);
+      if(co->name)
+        free(co->name);
+      if(co->value)
+        free(co->value);
+
+      free(co);
+      return NULL;
+    }
+
+  }
+
   /* now, we have parsed the incoming line, we must now check if this
      superceeds an already existing cookie, which it may if the previous have
      the same domain and path as this */
@@ -183,6 +282,11 @@ struct Cookie *cookie_add(struct CookieInfo *c,
         if(clist->expirestr)
           free(clist->expirestr);
 
+        if(clist->version)
+          free(clist->version);
+        if(clist->maxage)
+          free(clist->maxage);
+
         *clist = *co;  /* then store all the new data */
       }
 
@@ -212,30 +316,43 @@ struct CookieInfo *cookie_init(char *file)
 {
   char line[MAX_COOKIE_LINE];
   struct CookieInfo *c;
-  struct Cookie *co;
   FILE *fp;
-
+  bool fromfile=TRUE;
+  
   c = (struct CookieInfo *)malloc(sizeof(struct CookieInfo));
   if(!c)
     return NULL; /* failed to get memory */
   memset(c, 0, sizeof(struct CookieInfo));
   c->filename = strdup(file?file:"none"); /* copy the name just in case */
 
-  fp = file?fopen(file, "r"):NULL;
+  if(strequal(file, "-")) {
+    fp = stdin;
+    fromfile=FALSE;
+  }
+  else
+    fp = file?fopen(file, "r"):NULL;
+
   if(fp) {
     while(fgets(line, MAX_COOKIE_LINE, fp)) {
       if(strnequal("Set-Cookie:", line, 11)) {
         /* This is a cookie line, get it! */
         char *lineptr=&line[11];
-        while(*lineptr && isspace(*lineptr))
+        while(*lineptr && isspace((int)*lineptr))
           lineptr++;
 
-        co = cookie_add(c, lineptr);
-        if(!co)
-          break;
+        cookie_add(c, TRUE, lineptr);
+      }
+      else {
+        /* This might be a netscape cookie-file line, get it! */
+        char *lineptr=line;
+        while(*lineptr && isspace((int)*lineptr))
+          lineptr++;
+
+        cookie_add(c, FALSE, lineptr);
       }
     }
-    fclose(fp);
+    if(fromfile)
+      fclose(fp);
   }
 
   return c;
@@ -362,6 +479,11 @@ void cookie_cleanup(struct CookieInfo *c)
 	 if(co->expirestr)
 	    free(co->expirestr);
 
+	 if(co->version)
+	    free(co->version);
+	 if(co->maxage)
+	    free(co->maxage);
+
 	 next = co->next;
 	 free(co);
 	 co = next;
