@@ -24,14 +24,14 @@
  *
  * ------------------------------------------------------------
  * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
+ * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
  *
- * 	http://www.fts.frontec.se/~dast/curl/
+ * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/ftp.c,v $
+ * $Source: /cvsroot/curl/lib/ftp.c,v $
  * $Revision: 1.6 $
- * $Date: 1999/04/12 23:09:00 $
- * $Author: dast $
+ * $Date: 2000/03/16 11:39:31 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
@@ -44,7 +44,7 @@
 #include <ctype.h>
 #include <errno.h>
 
-#include "../include/setup.h"
+#include "setup.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -53,36 +53,37 @@
 #include <sys/select.h>
 #endif
 
-#if defined(WIN32) && !defined(__GNUC__)
+#if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
 #include <winsock.h>
 #else /* some kind of unix */
 #include <sys/socket.h>
 #include <netinet/in.h>
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 #include <sys/utsname.h>
 #include <netdb.h>
 #endif
 
-#if defined(WIN32) && defined(__GNUC__)
+#if defined(WIN32) && defined(__GNUC__) || defined(__MINGW32__)
 #include <errno.h>
 #endif
 
 
-#include "../include/curl.h"
+#include <curl/curl.h>
 #include "urldata.h"
 #include "sendf.h"
 
 #include "if2ip.h"
 #include "hostip.h"
 #include "progress.h"
-#include "upload.h"
 #include "download.h"
-
+#include "escape.h"
 
 /* returns last node in linked list */
-static struct slist *slist_get_last(struct slist *list)
+static struct curl_slist *slist_get_last(struct curl_slist *list)
 {
-	struct slist	*item;
+	struct curl_slist	*item;
 
 	/* if caller passed us a NULL, return now */
 	if (!list)
@@ -101,12 +102,12 @@ static struct slist *slist_get_last(struct slist *list)
  * function as well as an append function. If you find this bothersome,
  * then simply create a separate _init function and call it appropriately from
  * within the proram. */
-struct slist *curl_slist_append(struct slist *list, char *data)
+struct curl_slist *curl_slist_append(struct curl_slist *list, char *data)
 {
-	struct slist	*last;
-	struct slist	*new_item;
+	struct curl_slist	*last;
+	struct curl_slist	*new_item;
 
-	new_item = (struct slist *) malloc(sizeof(struct slist));
+	new_item = (struct curl_slist *) malloc(sizeof(struct curl_slist));
 	if (new_item) {
 		new_item->next = NULL;
 		new_item->data = strdup(data);
@@ -127,10 +128,10 @@ struct slist *curl_slist_append(struct slist *list, char *data)
 }
 
 /* be nice and clean up resources */
-void curl_slist_free_all(struct slist *list)
+void curl_slist_free_all(struct curl_slist *list)
 {
-	struct slist	*next;
-	struct slist	*item;
+	struct curl_slist	*next;
+	struct curl_slist	*item;
 
 	if (!list)
 		return;
@@ -230,9 +231,9 @@ static int GetLastResponse(int sockfd, char *buf,
     *ptr=0; /* zero terminate */
 
     if(data->conf & CONF_VERBOSE) {
-      fputs("< ", stderr);
-      fwrite(buf, 1, nread, stderr);
-      fputs("\n", stderr);
+      fputs("< ", data->err);
+      fwrite(buf, 1, nread, data->err);
+      fputs("\n", data->err);
     }
   } while(read_rc &&
 	  (nread<4 || !lastline(buf)) );
@@ -243,13 +244,7 @@ static int GetLastResponse(int sockfd, char *buf,
 char *getmyhost(void)
 {
   static char myhost[256];
-#if !defined(WIN32) && !defined(HAVE_UNAME) && !defined(HAVE_GETHOSTNAME)
-  /* We have no means of finding the local host name! */
-  strcpy(myhost, "localhost");
-#endif
-#if defined(WIN32) || !defined(HAVE_UNAME)
-  gethostname(myhost, 256);
-#else
+#ifdef HAVE_UNAME
   struct utsname ugnm;
 
   if (uname(&ugnm) < 0)
@@ -257,17 +252,70 @@ char *getmyhost(void)
 
   (void) strncpy(myhost, ugnm.nodename, 255);
   myhost[255] = '\0';
+#endif
+#ifdef HAVE_GETHOSTNAME
+  gethostname(myhost, 256);
+#endif
+#if !defined(HAVE_UNAME) && !defined(HAVE_GETHOSTNAME)
+  /* We have no means of finding the local host name! */
+  strcpy(myhost, "localhost");
 #endif
   return myhost;
 }
 
-/* -- deal with the ftp server!  -- */
+#if 0
+/*
+ * URLfix()
+ *
+ * This function returns a string converted FROM the input URL format to a
+ * format that is more likely usable for the remote server. That is, all
+ * special characters (found as %XX-codes) will be eascaped with \<letter>.
+ */
 
-UrgError ftp(struct UrlData *data,
-             long *bytecountp,
-             char *ftpuser,
-             char *ftppasswd,
-             char *ppath)
+static char *URLfix(char *string)
+{
+  /* The length of the new string can't be longer than twice the original
+     string, if all letters are '+'... */
+  int alloc = strlen(string)*2;
+  char *ns = malloc(alloc);
+  unsigned char in;
+  int index=0;
+  int hex;
+   
+  while(*string) {
+    in = *string;
+    switch(in) {
+    case '+':
+      ns[index++] = '\\';
+      ns[index++] = ' ';
+      string++;
+      continue;
+
+    case '%':
+      /* encoded part */
+      if(sscanf(string+1, "%02X", &hex)) {
+        ns[index++] = '\\';
+        ns[index++] = hex;
+        string+=3;
+        continue;
+      }
+      /* FALLTHROUGH */
+    default:
+      ns[index++] = in;
+      string++;
+    }
+  }
+  ns[index]=0; /* terminate it */
+  return ns;
+}
+#endif
+
+static
+UrgError _ftp(struct UrlData *data,
+              long *bytecountp,
+              char *ftpuser,
+              char *ftppasswd,
+              char *ppath)
 {
   /* this is FTP and no proxy */
   size_t nread;
@@ -277,7 +325,7 @@ UrgError ftp(struct UrlData *data,
   int portsock=-1;
   struct sockaddr_in serv_addr;
 
-  struct slist *qitem; /* QUOTE item */
+  struct curl_slist *qitem; /* QUOTE item */
 
   /* The first thing we do is wait for the "220*" line: */
   nread = GetLastResponse(data->firstsocket, buf, data);
@@ -331,6 +379,63 @@ UrgError ftp(struct UrlData *data,
     return URG_FTP_WEIRD_USER_REPLY;
   }
 
+  /* Send any QUOTE strings? */
+  if(data->quote) {
+    qitem = data->quote;
+    /* Send all QUOTE strings in same order as on command-line */
+    while (qitem) {
+      /* Send string */
+      if (qitem->data) {
+        sendf(data->firstsocket, data, "%s\r\n", qitem->data);
+
+        nread = GetLastResponse(data->firstsocket, buf, data);
+
+        if (buf[0] != '2') {
+          failf(data, "QUOT string not accepted: %s",
+                qitem->data);
+          return URG_FTP_QUOTE_ERROR;
+        }
+      }
+      qitem = qitem->next;
+    }
+  }
+
+  /* If we have selected NOBODY, it means that we only want file information.
+     Which in FTP can't be much more than the file size! */
+  if(data->conf & CONF_NOBODY) {
+    /* The SIZE command is _not_ RFC 959 specified, and therefor many servers
+       may not support it! It is however the only way we have to get a file's
+       size! */
+    int filesize;
+    sendf(data->firstsocket, data, "SIZE %s\r\n", ppath);
+
+    nread = GetLastResponse(data->firstsocket, buf, data);
+
+    if(strncmp(buf, "213", 3)) {
+      failf(data, "Couldn't get file size: %s", buf+4);
+      return URG_FTP_COULDNT_GET_SIZE;
+    }
+    /* get the size from the ascii string: */
+    filesize = atoi(buf+4);
+
+    sprintf(buf, "Content-Length: %d\n", filesize);
+
+    if(strlen(buf) != data->fwrite(buf, 1, strlen(buf), data->out)) {
+      failf (data, "Failed writing output");
+      return URG_WRITE_ERROR;
+    }
+    if(data->writeheader) {
+      /* the header is requested to be written to this file */
+      if(strlen(buf) != data->fwrite (buf, 1, strlen(buf),
+                                      data->writeheader)) {
+        failf (data, "Failed writing output");
+        return URG_WRITE_ERROR;
+      }
+    }
+    return URG_OK;
+  }
+
+  /* We have chosen to use the PORT command */
   if(data->conf & CONF_FTPPORT) {
     struct sockaddr_in sa;
     struct hostent *h=NULL;
@@ -418,7 +523,7 @@ UrgError ftp(struct UrlData *data,
       return URG_FTP_PORT_FAILED;
     }     
   }
-  else {
+  else { /* we use the PASV command */
 
     sendf(data->firstsocket, data, "PASV\r\n");
 
@@ -434,19 +539,29 @@ UrgError ftp(struct UrlData *data,
       unsigned short newport;
       char newhost[32];
       struct hostent *he;
-      /* 227 Entering Passive Mode (127,0,0,1,4,51) */
-      if(6 != sscanf(buf, "%*[^(](%d,%d,%d,%d,%d,%d)",
-                     &ip[0], &ip[1], &ip[2], &ip[3],
-                     &port[0], &port[1])) {
+      char *str=buf;
 
-        /* MVS ftpd returns a different 227 string */
-        if (6 != sscanf(buf, "227 Data transfer will passively listen to %d,%d,%d,%d,%d,%d",
-                        &ip[0], &ip[1], &ip[2], &ip[3],
-                        &port[0], &port[1])) {
-
-          failf(data, "Oddly formatted 227-reply: %s", buf);
-          return URG_FTP_WEIRD_227_FORMAT;
-        }
+      /*
+       * New 227-parser June 3rd 1999.
+       * It now scans for a sequence of six comma-separated numbers and
+       * will take them as IP+port indicators.
+       *
+       * Found reply-strings include:
+       * "227 Entering Passive Mode (127,0,0,1,4,51)"
+       * "227 Data transfer will passively listen to 127,0,0,1,4,51"
+       * "227 Entering passive mode. 127,0,0,1,4,51"
+       */
+      
+      while(*str) {
+	 if (6 == sscanf(str, "%d,%d,%d,%d,%d,%d",
+			 &ip[0], &ip[1], &ip[2], &ip[3],
+			 &port[0], &port[1]))
+	    break;
+	 str++;
+      }
+      if(!*str) {
+	 failf(data, "Couldn't interpret this 227-reply: %s", buf);
+	 return URG_FTP_WEIRD_227_FORMAT;
       }
       sprintf(newhost, "%d.%d.%d.%d", ip[0], ip[1], ip[2], ip[3]);
       he = GetHost(data, newhost);
@@ -470,7 +585,7 @@ UrgError ftp(struct UrlData *data,
         struct hostent * answer;
 
         unsigned long address;
-#if defined(HAVE_INET_ADDR) || defined(WIN32)
+#if defined(HAVE_INET_ADDR)
         address = inet_addr(newhost);
         answer = gethostbyaddr((char *) &address, sizeof(address), 
                                AF_INET);
@@ -512,27 +627,6 @@ UrgError ftp(struct UrlData *data,
   }
   /* we have the (new) data connection ready */
 
-  /* Send any QUOTE strings? */
-  if(data->quote) {
-    qitem = data->quote;
-    /* Send all QUOTE strings in same order as on command-line */
-    while (qitem) {
-      /* Send string */
-      if (qitem->data) {
-        sendf(data->firstsocket, data, "%s\r\n", qitem->data);
-
-        nread = GetLastResponse(data->firstsocket, buf, data);
-
-        if (strncmp(buf, "200", 3)) {
-          failf(data, "QUOT string not accepted: %s",
-                qitem->data);
-          return URG_FTP_QUOTE_ERROR;
-        }
-      }
-      qitem = qitem->next;
-    }
-  }
-
   if(data->conf & CONF_UPLOAD) {
 
     /* Set type to binary (unless specified ASCII) */
@@ -660,20 +754,59 @@ UrgError ftp(struct UrlData *data,
     /* When we know we're uploading a specified file, we can get the file
        size prior to the actual upload. */
 
+    pgrsSetUploadSize(data, data->infilesize);
+#if 0
     ProgressInit(data, data->infilesize);
-    result = Upload(data, data->secondarysocket, bytecountp);
+#endif
+    result = Transfer(data, -1, -1, FALSE, NULL, /* no download */
+                      data->secondarysocket, bytecountp);
     if(result)
       return result;
       
     if((-1 != data->infilesize) && (data->infilesize != *bytecountp)) {
       failf(data, "Wrote only partial file (%d out of %d bytes)",
             *bytecountp, data->infilesize);
-      return URG_FTP_PARTIAL_FILE;
+      return URG_PARTIAL_FILE;
     }
   }
   else {
     /* Retrieve file or directory */
     bool dirlist=FALSE;
+    long downloadsize=-1;
+
+    if(data->conf&CONF_RANGE && data->range) {
+      int from, to;
+      int totalsize=-1;
+      char *ptr;
+      char *ptr2;
+
+      from=strtol(data->range, &ptr, 0);
+      while(ptr && *ptr && (isspace((int)*ptr) || (*ptr=='-')))
+        ptr++;
+      to=strtol(ptr, &ptr2, 0);
+      if(ptr == ptr2) {
+        /* we didn't get any digit */
+        to=-1;
+      }
+      if(-1 == to) {
+        /* X - */
+        data->resume_from = from;
+      }
+      else if(from < 0) {
+        /* -Y */
+        from = 0;
+        to = -from;
+        totalsize = to-from;
+        data->maxdownload = totalsize;
+      }
+      else {
+        /* X- */
+        totalsize = to-from;
+        data->maxdownload = totalsize;
+      }
+      infof(data, "range-download from %d to %d, totally %d bytes\n",
+            from, to, totalsize);
+    }
 
     if(!ppath[0])
       /* make sure this becomes a valid name */
@@ -701,7 +834,8 @@ UrgError ftp(struct UrlData *data,
          standard in any way */
 
       sendf(data->firstsocket, data, "%s %s\r\n",
-            data->conf&CONF_FTPLISTONLY?"NLST":"LIST",
+            data->customrequest?data->customrequest:
+            (data->conf&CONF_FTPLISTONLY?"NLST":"LIST"),
             ppath);
     }
     else {
@@ -719,6 +853,37 @@ UrgError ftp(struct UrlData *data,
       }
 
       if(data->resume_from) {
+
+        /* Daniel: (August 4, 1999)
+         *
+         * We start with trying to use the SIZE command to figure out the size
+         * of the file we're gonna get. If we can get the size, this is by far
+         * the best way to know if we're trying to resume beyond the EOF.  */
+
+        sendf(data->firstsocket, data, "SIZE %s\r\n", ppath);
+
+        nread = GetLastResponse(data->firstsocket, buf, data);
+
+        if(strncmp(buf, "213", 3)) {
+          infof(data, "server doesn't support SIZE: %s", buf+4);
+          /* We couldn't get the size and therefore we can't know if there
+             really is a part of the file left to get, although the server
+             will just close the connection when we start the connection so it
+             won't cause us any harm, just not make us exit as nicely. */
+        }
+        else {
+          int foundsize=atoi(buf+4);
+          /* We got a file size report, so we check that there actually is a
+             part of the file left to get, or else we go home.  */
+          if(foundsize <= data->resume_from) {
+            failf(data, "Offset (%d) was beyond file size (%d)",
+                  data->resume_from, foundsize);
+            return URG_FTP_BAD_DOWNLOAD_RESUME;
+          }
+          /* Now store the number of bytes we are expected to download */
+          downloadsize = foundsize-data->resume_from;
+        }
+
         /* Set resume file transfer offset */
         infof(data, "Instructs server to resume from offset %d\n",
               data->resume_from);
@@ -759,7 +924,7 @@ UrgError ftp(struct UrlData *data,
 
       int size=-1; /* default unknown size */
 
-      if(!dirlist) {
+      if(!dirlist && (-1 == downloadsize)) {
         /*
          * It seems directory listings either don't show the size or very
          * often uses size 0 anyway.
@@ -775,7 +940,7 @@ UrgError ftp(struct UrlData *data,
             if('(' == *bytes)
               break;
             /* if only skip digits, or else we're in deep trouble */
-            if(!isdigit(*bytes)) {
+            if(!isdigit((int)*bytes)) {
               bytes=NULL;
               break;
             }
@@ -794,15 +959,19 @@ UrgError ftp(struct UrlData *data,
           size=-1;
 #endif
       }
+      else if(downloadsize > -1)
+        size = downloadsize;
 
+#if 0
       if((size > -1) && (data->resume_from>0)) {
         size -= data->resume_from;
         if(size <= 0) {
           failf(data, "Offset (%d) was beyond file size (%d)",
                 data->resume_from, data->resume_from+size);
-          return URG_FTP_PARTIAL_FILE;
+          return URG_PARTIAL_FILE;
         }
       }
+#endif
 
       if(data->conf & CONF_FTPPORT) {
         result = AllowServerConnect(data, portsock);
@@ -810,19 +979,18 @@ UrgError ftp(struct UrlData *data,
           return result;
       }
 
-      ProgressInit(data, size);
-
       infof(data, "Getting file with size: %d\n", size);
 
       /* FTP download: */
-      result=Download(data, data->secondarysocket, size, FALSE,
-                      bytecountp);
+      result=Transfer(data, data->secondarysocket, size, FALSE,
+                      bytecountp,
+                      -1, NULL); /* no upload here */
       if(result)
         return result;
 
       if((-1 != size) && (size != *bytecountp)) {
         failf(data, "Received only partial file");
-        return URG_FTP_PARTIAL_FILE;
+        return URG_PARTIAL_FILE;
       }
       else if(0 == *bytecountp) {
         failf(data, "No data was received!");
@@ -836,7 +1004,10 @@ UrgError ftp(struct UrlData *data,
 	
   }
   /* end of transfer */
+#if 0
   ProgressEnd(data);
+#endif
+  pgrsDone(data);
 
   /* shut down the socket to inform the server we're done */
   sclose(data->secondarysocket);
@@ -852,5 +1023,55 @@ UrgError ftp(struct UrlData *data,
     return URG_FTP_WRITE_ERROR;
   }
 
+  /* Send any post-transfer QUOTE strings? */
+  if(data->postquote) {
+    qitem = data->postquote;
+    /* Send all QUOTE strings in same order as on command-line */
+    while (qitem) {
+      /* Send string */
+      if (qitem->data) {
+        sendf(data->firstsocket, data, "%s\r\n", qitem->data);
+
+        nread = GetLastResponse(data->firstsocket, buf, data);
+
+        if (buf[0] != '2') {
+          failf(data, "QUOT string not accepted: %s",
+                qitem->data);
+          return URG_FTP_QUOTE_ERROR;
+        }
+      }
+      qitem = qitem->next;
+    }
+  }
+
+
   return URG_OK;
 }
+
+/* -- deal with the ftp server!  -- */
+
+UrgError ftp(struct UrlData *data,
+             long *bytecountp,
+             char *ftpuser,
+             char *ftppasswd,
+             char *urlpath)
+{
+  char *realpath;
+  UrgError retcode;
+
+#if 0
+  realpath = URLfix(urlpath);
+#else
+  realpath = curl_unescape(urlpath);
+#endif
+  if(realpath) {
+    retcode = _ftp(data, bytecountp, ftpuser, ftppasswd, realpath);
+    free(realpath);
+  }
+  else
+    /* then we try the original path */
+    retcode = _ftp(data, bytecountp, ftpuser, ftppasswd, urlpath);
+
+  return retcode;
+}
+
