@@ -21,7 +21,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  * ============================================================================
  *
- * $Id: getpass.c,v 1.1.1.1 1999/03/11 22:23:34 dast Exp $
+ * $Id: getpass.c,v 1.3 2000/03/19 19:54:13 bagder Exp $
  *
  * The spirit of this license is to allow use of this source code in any
  * project be it open or closed but still encourage the use of the open,
@@ -31,7 +31,7 @@
  *   Angus Mackay <amackay@gus.ml.org>
  *
  * Contributor(s):
- *   Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
+ *   Daniel Stenberg <Daniel.Stenberg@haxx.nu>
  */
 
 #ifndef WIN32
@@ -76,7 +76,9 @@ char *getpass(const char *prompt)
   FILE *outfp;
   static char buf[INPUT_BUFFER];
   RETSIGTYPE (*sigint)();
+#ifndef __EMX__
   RETSIGTYPE (*sigtstp)();
+#endif
   size_t bytes_read;
   int infd;
   int outfd;
@@ -92,7 +94,11 @@ char *getpass(const char *prompt)
 #endif
 
   sigint = signal(SIGINT, SIG_IGN);
+  /* 20000318 mgs
+   * this is needed by the emx system, SIGTSTP is not a supported signal */
+#ifndef __EMX__
   sigtstp = signal(SIGTSTP, SIG_IGN);
+#endif
 
   if( (infp=fopen("/dev/tty", "r")) == NULL )
   {
@@ -173,5 +179,13 @@ char *getpass(const char *prompt)
 
   return(buf);
 }
-
+#else
+#include <stdio.h>
+char *getpass(const char *prompt)
+{
+	static char password[80];
+	printf(prompt);
+	gets(password);
+	return password;
+}
 #endif /* don't do anything if WIN32 */
