@@ -28,10 +28,10 @@
  *
  * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/hostip.c,v $
- * $Revision: 1.7 $
- * $Date: 1999/09/06 06:59:40 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/lib/hostip.c,v $
+ * $Revision: 1.2 $
+ * $Date: 2000/01/10 23:36:14 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
@@ -40,7 +40,7 @@
 
 #include <string.h>
 
-#include "../include/setup.h"
+#include "setup.h"
 
 #if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
 #include <winsock.h>
@@ -53,8 +53,10 @@
 #endif
 #include <netinet/in.h>
 #include <netdb.h>
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
 #endif
+#endif
 
 #include "urldata.h"
 #include "sendf.h"
