@@ -24,14 +24,14 @@
  *
  * ------------------------------------------------------------
  * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
+ * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
  *
- * 	http://www.fts.frontec.se/~dast/curl/
+ * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/if2ip.c,v $
- * $Revision: 1.4 $
- * $Date: 1999/04/05 15:49:28 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/lib/if2ip.c,v $
+ * $Revision: 1.2 $
+ * $Date: 2000/01/10 23:36:14 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
@@ -42,19 +42,23 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "../include/setup.h"
+#include "setup.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#ifndef WIN32
+#if ! defined(WIN32) && ! defined(__BEOS__)
 
 #include <sys/socket.h>
 #include <netinet/in.h>
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
 #include <netinet/in.h>
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 #include <sys/ioctl.h>
 
 /* -- if2ip() -- */
