@@ -26,12 +26,12 @@
  * Contributor(s):
  *  Bjrn Reese <breese@mail1.stofanet.dk>
  *
- * 	http://www.fts.frontec.se/~dast/curl/
+ * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/ldap.c,v $
- * $Revision: 1.1 $
- * $Date: 1999/09/06 06:59:27 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/lib/ldap.c,v $
+ * $Revision: 1.2 $
+ * $Date: 2000/01/10 23:36:14 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
@@ -48,7 +48,7 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-#include "../include/setup.h"
+#include "setup.h"
 
 #if defined(WIN32) && !defined(__GNUC__)
 #else
@@ -61,12 +61,12 @@
 #endif
 
 #include "urldata.h"
-#include "../include/curl.h"
+#include <curl/curl.h>
 #include "sendf.h"
 #include "escape.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
-#include "../include/mprintf.h"
+#include <curl/mprintf.h>
 
 
 #define DYNA_GET_FUNCTION(type, fnc) \
@@ -125,6 +125,7 @@ static int WriteProc(void *param, char *text, int len)
   struct UrlData *data = (struct UrlData *)param;
   
   printf("%s\n", text);
+  return 0;
 }
 
 /***********************************************************************
@@ -145,11 +146,13 @@ UrgError ldap(struct UrlData *data, char *path, long *bytecount)
   void *server;
   void *result;
   void *entryIterator;
+#if 0
   char *dn;
   char **attrArray;
   char *attrIterator;
   char *attrString;
   void *dummy;
+#endif
   int ldaptext;
   
   infof(data, "LDAP: %s %s\n", data->url);
