@@ -28,40 +28,33 @@
  *   Linas Vepstas <linas@linas.org>
  *   Bjorn Reese <breese@imada.ou.dk>
  *   Johan Anderson <johan@homemail.com>
- *   Kjell Ericson <Kjell.Ericson@sth.frontec.se>
+ *   Kjell Ericson <Kjell.Ericson@haxx.nu>
  *   Troy Engel <tengel@palladium.net>
  *   Ryan Nelson <ryan@inch.com>
- *   Bjorn Stenberg <Bjorn.Stenberg@sth.frontec.se>
+ *   Bjorn Stenberg <Bjorn.Stenberg@haxx.nu>
  *   Angus Mackay <amackay@gus.ml.org>
  *
  * ------------------------------------------------------------
  * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
+ * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
  *
- * 	http://www.fts.frontec.se/~dast/curl/
+ * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/netrc.c,v $
+ * $Source: /cvsroot/curl/lib/netrc.c,v $
  * $Revision: 1.2 $
- * $Date: 1999/03/13 10:49:53 $
- * $Author: dast $
+ * $Date: 2000/01/10 23:36:15 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
  * ------------------------------------------------------------
- * $Log: netrc.c,v $
- * Revision 1.2  1999/03/13 10:49:53  dast
- * win32-adjusted
- *
- * Revision 1.1.1.1  1999/03/11 22:23:34  dast
- * Imported sources
- *
  ****************************************************************************/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include "../include/setup.h"
+#include "setup.h"
 #include "getenv.h"
 
 /* Debug this single source file with:
