@@ -28,10 +28,10 @@
  *
  * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/sendf.c,v $
- * $Revision: 1.7 $
- * $Date: 1999/09/06 06:59:41 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/lib/sendf.c,v $
+ * $Revision: 1.2 $
+ * $Date: 2000/01/10 23:36:15 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
@@ -42,7 +42,7 @@
 #include <stdarg.h>
 #include <stdlib.h>
 
-#include "../include/setup.h"
+#include "setup.h"
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -51,10 +51,10 @@
 #include <winsock.h>
 #endif
 
-#include "../include/curl.h"
+#include <curl/curl.h>
 #include "urldata.h"
 
-#include "../include/mprintf.h"
+#include <curl/mprintf.h>
 
 /* infof() is for info message along the way */
 
@@ -63,8 +63,8 @@ void infof(struct UrlData *data, char *fmt, ...)
   va_list ap;
   if(data->conf & CONF_VERBOSE) {
     va_start(ap, fmt);
-    fputs("* ", stderr);
-    vfprintf(stderr, fmt, ap);
+    fputs("* ", data->err);
+    vfprintf(data->err, fmt, ap);
     va_end(ap);
   }
 }
@@ -78,8 +78,8 @@ void failf(struct UrlData *data, char *fmt, ...)
   va_start(ap, fmt);
   if(data->errorbuffer)
     vsprintf(data->errorbuffer, fmt, ap);
-  else /* no errorbuffer receives this, write to stderr instead */
-    vfprintf(stderr, fmt, ap);
+  else /* no errorbuffer receives this, write to data->err instead */
+    vfprintf(data->err, fmt, ap);
   va_end(ap);
 }
 
@@ -96,7 +96,7 @@ int sendf(int fd, struct UrlData *data, char *fmt, ...)
   if(!s)
     return 0; /* failure */
   if(data->conf & CONF_VERBOSE)
-    fprintf(stderr, "> %s", s);
+    fprintf(data->err, "> %s", s);
 #ifndef USE_SSLEAY
    bytes_written = swrite(fd, s, strlen(s));
 #else
