@@ -28,10 +28,10 @@
  *
  * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/speedcheck.c,v $
- * $Revision: 1.6 $
- * $Date: 1999/09/06 06:59:41 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/lib/speedcheck.c,v $
+ * $Revision: 1.2 $
+ * $Date: 2000/01/10 23:36:15 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
@@ -43,7 +43,7 @@
 #include <winsock.h>
 #endif
 
-#include "../include/curl.h"
+#include <curl/curl.h>
 #include "urldata.h"
 #include "sendf.h"
 #include "speedcheck.h"
@@ -76,12 +76,6 @@ UrgError speedcheck(struct UrlData *data,
     /* we keep up the required speed all right */
     keeps_speed = now;
   }
-#if 0
-  if (data->conf & CONF_VERBOSE) {
-    fprintf(stderr, "* %lf > %d\n",
-	    data->current_speed, data->low_speed_limit);
-  }
-#endif
   return URG_OK;
 }
 
