@@ -30,17 +30,17 @@
  *
  * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/speedcheck.h,v $
- * $Revision: 1.5 $
- * $Date: 1999/09/06 06:59:41 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/lib/speedcheck.h,v $
+ * $Revision: 1.2 $
+ * $Date: 2000/01/10 23:36:15 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
  * ------------------------------------------------------------
  ****************************************************************************/
 
-#include "../include/setup.h"
+#include "setup.h"
 
 #include "timeval.h"
 
