@@ -24,20 +24,23 @@
  *
  * ------------------------------------------------------------
  * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
+ * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
  *
- * 	http://www.fts.frontec.se/~dast/curl/
+ * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/timeval.c,v $
- * $Revision: 1.1 $
- * $Date: 1999/04/05 15:51:00 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/lib/timeval.c,v $
+ * $Revision: 1.2 $
+ * $Date: 2000/01/10 23:36:15 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
  * ------------------------------------------------------------
  ****************************************************************************/
 
+#ifdef WIN32
+#include <windows.h>
+#endif
 #include "timeval.h"
 
 #ifndef HAVE_GETTIMEOFDAY
