@@ -22,28 +22,16 @@
  *  Portions created by the Initial Developer are Copyright (C) 1998.
  *  All Rights Reserved.
  *
- *  Contributor(s):
- *   Rafael Sagula <sagula@inf.ufrgs.br>
- *   Sampo Kellomaki <sampo@iki.fi>
- *   Linas Vepstas <linas@linas.org>
- *   Bjorn Reese <breese@imada.ou.dk>
- *   Johan Anderson <johan@homemail.com>
- *   Kjell Ericson <Kjell.Ericson@sth.frontec.se>
- *   Troy Engel <tengel@palladium.net>
- *   Ryan Nelson <ryan@inch.com>
- *   Bjorn Stenberg <Bjorn.Stenberg@sth.frontec.se>
- *   Angus Mackay <amackay@gus.ml.org>
- *
  * ------------------------------------------------------------
  * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
+ * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
  *
- * 	http://www.fts.frontec.se/~dast/curl/
+ * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/url.c,v $
- * $Revision: 1.10 $
- * $Date: 1999/04/19 22:22:00 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/lib/url.c,v $
+ * $Revision: 1.9 $
+ * $Date: 2000/03/01 22:05:11 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
@@ -66,9 +54,9 @@
 
 #include <errno.h>
 
-#include "../include/setup.h"
+#include "setup.h"
 
-#if defined(WIN32) && !defined(__GNUC__)
+#if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
 #include <winsock.h>
 #include <time.h>
 #include <io.h>
@@ -83,8 +71,12 @@
 #include <unistd.h>
 #endif
 #include <netdb.h>
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 #include <sys/ioctl.h>
 #include <signal.h>
 
@@ -109,7 +101,7 @@
 #endif
 
 #include "urldata.h"
-#include "../include/curl.h"
+#include <curl/curl.h>
 #include "netrc.h"
 
 #include "formdata.h"
@@ -118,24 +110,30 @@
 #include "ssluse.h"
 #include "hostip.h"
 #include "if2ip.h"
-#include "upload.h"
 #include "download.h"
 #include "sendf.h"
 #include "speedcheck.h"
 #include "getpass.h"
-#include "ftp.h"
 #include "progress.h"
 #include "cookie.h"
 
+/* And now for the protocols */
+#include "ftp.h"
+#include "dict.h"
+#include "telnet.h"
+#include "http.h"
+#include "file.h"
+#include "ldap.h"
+
 #define _MPRINTF_REPLACE /* use our functions only */
-#include "../include/mprintf.h"
+#include <curl/mprintf.h>
 
 /* -- -- */
 
 /***********************************************************************
  * Start with some silly functions to make win32-systems survive
  ***********************************************************************/
-#if defined(WIN32) && !defined(__GNUC__)
+#if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
 static void cleanup(void)
 {
   WSACleanup();
@@ -182,6 +180,23 @@ static UrgError _urlget(struct UrlData *data);
 
 void urlfree(struct UrlData *data, bool totally)
 {
+#ifdef USE_SSLEAY
+  if (data->use_ssl) {
+    if(data->ssl) {
+      SSL_shutdown(data->ssl);
+      SSL_set_connect_state(data->ssl);
+
+      SSL_free (data->ssl);
+      data->ssl = NULL;
+    }
+    if(data->ctx) {
+      SSL_CTX_free (data->ctx);
+      data->ctx = NULL;
+    }
+    data->use_ssl = FALSE; /* get back to ordinary socket usage */
+  }
+#endif /* USE_SSLEAY */
+
   /* close possibly still open sockets */
   if(-1 != data->secondarysocket) {
     sclose(data->secondarysocket);
@@ -192,14 +207,6 @@ void urlfree(struct UrlData *data, bool totally)
     data->firstsocket=-1;
   }
 
-#ifdef USE_SSLEAY
-  if (data->use_ssl) {
-    if(data->ssl)
-      SSL_free (data->ssl);
-    if(data->ctx)
-      SSL_CTX_free (data->ctx);
-  }
-#endif /* USE_SSLEAY */
 
   if(data->ptr_proxyuserpwd) {
     free(data->ptr_proxyuserpwd);
@@ -225,6 +232,10 @@ void urlfree(struct UrlData *data, bool totally)
     free(data->ptr_cookie);
     data->ptr_cookie=NULL;
   }
+  if(data->ptr_host) {
+    free(data->ptr_host);
+    data->ptr_host=NULL;
+  }
 
   if(totally) {
     /* we let the switch decide whether we're doing a part or total
@@ -271,6 +282,7 @@ UrgError curl_urlget(UrgTag tag, ...)
     /* Let's set some default values: */
     data->out = stdout; /* default output to stdout */
     data->in  = stdin;  /* default input from stdin */
+    data->err  = stderr;  /* default stderr to stderr */
     data->firstsocket = -1; /* no file descriptor */
     data->secondarysocket = -1; /* no file descriptor */
 
@@ -314,8 +326,16 @@ UrgError curl_urlget(UrgTag tag, ...)
         data->moredoc = (struct MoreDoc *)param_obj;
         break;
 #endif
+      case URGTAG_TIMECONDITION:
+        data->timecondition = (long)param_long;
+        break;
+
+      case URGTAG_TIMEVALUE:
+        data->timevalue = (long)param_long;
+        break;
+
       case URGTAG_SSLVERSION:
-        data->ssl_version = (int)param_obj;
+        data->ssl_version = (int)param_long;
         break;
 
       case URGTAG_COOKIEFILE:
@@ -342,6 +362,9 @@ UrgError curl_urlget(UrgTag tag, ...)
       case URGTAG_HTTPHEADER:
 	data->headers = (struct HttpHeader *)param_obj;
 	break;
+      case URGTAG_CUSTOMREQUEST:
+	data->customrequest = (char *)param_obj;
+	break;
       case URGTAG_HTTPPOST:
 	data->httppost = (struct HttpPost *)param_obj;
 	break;
@@ -368,6 +391,9 @@ UrgError curl_urlget(UrgTag tag, ...)
       case URGTAG_POSTFIELDS:
         data->postfields = (char *)param_obj;
         break;
+      case URGTAG_PROGRESSMODE:
+        data->progress.mode = (long)param_long;
+        break;
       case URGTAG_REFERER:
         data->referer = (char *)param_obj;
         break;
@@ -386,6 +412,9 @@ UrgError curl_urlget(UrgTag tag, ...)
       case URGTAG_USERPWD:
         data->userpwd = (char *)param_obj;
         break;
+      case URGTAG_POSTQUOTE:
+        data->postquote = (struct curl_slist *)param_obj;
+        break;
       case URGTAG_PROXYUSERPWD:
         data->proxyuserpwd = (char *)param_obj;
         break;
@@ -395,9 +424,15 @@ UrgError curl_urlget(UrgTag tag, ...)
       case URGTAG_RESUME_FROM:
 	data->resume_from = (long)param_long;
 	break;
+      case URGTAG_STDERR:
+	data->err = (FILE *)param_obj;
+	break;
       case URGTAG_WRITEFUNCTION:
         data->fwrite = (size_t (*)(char *, size_t, size_t, FILE *))param_func;
         break;
+      case URGTAG_WRITEINFO:
+        data->writeinfo = (char *)param_obj;
+        break;
       case URGTAG_READFUNCTION:
         data->fread = (size_t (*)(char *, size_t, size_t, FILE *))param_func;
         break;
@@ -411,7 +446,7 @@ UrgError curl_urlget(UrgTag tag, ...)
 	data->crlf = (long)param_long;
 	break;
       case URGTAG_QUOTE:
-        data->quote = (struct slist *)param_obj;
+        data->quote = (struct curl_slist *)param_obj;
         break;
       case URGTAG_DONE: /* done with the parsing, fall through */
         continue;
@@ -424,6 +459,9 @@ UrgError curl_urlget(UrgTag tag, ...)
 
     va_end(arg);
 
+    pgrsMode(data, data->progress.mode);
+    pgrsStartNow(data);
+
     data-> headerbuff=(char*)malloc(HEADERSIZE);
     if(!data->headerbuff)
       return URG_FAILED_INIT;
@@ -434,8 +472,6 @@ UrgError curl_urlget(UrgTag tag, ...)
 
     while((res == URG_OK) && data->newurl) {
       /* Location: redirect */
-      char *freethis = data->newurl;
-
       char prot[16];
       char path[URL_MAX_LENGTH];
 
@@ -444,7 +480,6 @@ UrgError curl_urlget(UrgTag tag, ...)
 	/***
 	 *DANG* this is an RFC 2068 violation. The URL is supposed
 	 to be absolute and this doesn't seem to be that!
-	 At least the Zeus HTTP server seem to do this.
 	 ***
 	 Instead, we have to TRY to append this new path to the old URL
 	 to the right of the host part. Oh crap, this is doomed to cause
@@ -452,17 +487,25 @@ UrgError curl_urlget(UrgTag tag, ...)
 	 */
 	char *protsep;
 	char *pathsep;
-	int length;
 	char *newest;
 
 	/* protsep points to the start of the host name */
 	protsep=strstr(data->url, "//");
 	if(!protsep)
 	  protsep=data->url;
-	else
+	else {
+          data->port=0; /* we got a full URL and then we should reset the
+                           port number here to re-initiate it later */
 	  protsep+=2; /* pass the // */
+        }
 
         if('/' != data->newurl[0]) {
+          /* First we need to find out if there's a ?-letter in the URL, and
+             cut it and the right-side of that off */
+          pathsep = strrchr(protsep, '?');
+          if(pathsep)
+            *pathsep=0;
+
           /* we have a relative path to append to the last slash if
              there's one available */
           pathsep = strrchr(protsep, '/');
@@ -477,10 +520,14 @@ UrgError curl_urlget(UrgTag tag, ...)
             *pathsep=0;
         }
 
-	newest = maprintf("%s%s%s", data->url, ('/' == data->newurl[0])?"":"/",
-			  data->newurl);
+        newest=(char *)malloc( strlen(data->url) +
+                               1 + /* possible slash */
+                               strlen(data->newurl) + 1/* zero byte */);
+
 	if(!newest)
 	  return URG_OUT_OF_MEMORY;
+        sprintf(newest, "%s%s%s", data->url, ('/' == data->newurl[0])?"":"/",
+                data->newurl);
 	free(data->newurl);
 	data->newurl = newest;
       }
@@ -494,13 +541,20 @@ UrgError curl_urlget(UrgTag tag, ...)
       urlfree(data, FALSE);
 
       res = _urlget(data);
-      free(freethis);
     }
+    if(data->newurl)
+      free(data->newurl);
 
   }
   else
     res = URG_FAILED_INIT; /* failed */
 
+  if((URG_OK == res) && data->writeinfo) {
+    /* Time to output some info to stdout */
+    WriteOut(data);
+  }
+
+
   /* total cleanup */
   urlfree(data, TRUE);
 
@@ -540,9 +594,9 @@ static int GetLine(int sockfd, char *buf,
   *ptr=0; /* zero terminate */
 
   if(data->conf & CONF_VERBOSE) {
-    fputs("< ", stderr);
-    fwrite(buf, 1, nread, stderr);
-    fputs("\n", stderr);
+    fputs("< ", data->err);
+    fwrite(buf, 1, nread, data->err);
+    fputs("\n", data->err);
   }
   return nread;
 }
@@ -561,46 +615,6 @@ RETSIGTYPE alarmfunc(int signal)
 }
 #endif
 
-#ifdef TRY_TELNET
-/* for telnet testing only */
-
-char *telnetin(struct UrlData *data,
-	       unsigned char inbyte)
-{
-#define TELNET_SIZE 256
-  static char inbuffer[TELNET_SIZE+1];
-  static int index=0;
-
-  if(10 == inbyte) {
-    /* newline, we reset buffer */
-    index=0;
-    inbuffer[0]=0;
-#if 0
-    fprintf(stderr, "[NOLL]\n");
-#endif
-  }
-  else if(' ' > inbyte) {
-    /* ignore low ASCII numbers */
-#if 0
-    fprintf(stderr, "[IGNORE] %u\n", inbyte);
-#endif
-  }
-  else if(index < TELNET_SIZE) {
-#if 0
-    fprintf(stderr, "In:%c\n", inbyte);
-#endif
-    inbuffer[index]=inbyte;
-    inbuffer[index+1]=0; /* zero terminate all times to allow string
-			    functions to operate on buffer */
-    index++;
-  }
-  /* else
-     buffer is full, skip it */
-  return inbuffer;
-}
-
-#endif
-
 /* ====================================================== */
 /*
  * urlget <url>
@@ -627,17 +641,13 @@ static UrgError _urlget(struct UrlData *data)
   char *ppath, *tmp;
   long bytecount;
   struct timeval now;
-  char ftpuser[128]="";
-  char ftppasswd[128]="";
-  char proxyuser[128]="";
-  char proxypasswd[128]="";
 
   UrgError result;
   char resumerange[12]="";
 
   buf = data->buffer; /* this is our buffer */
 
-#ifndef WIN32
+#if 0
   signal(SIGALRM, alarmfunc);
 #endif
 
@@ -647,8 +657,29 @@ static UrgError _urlget(struct UrlData *data)
    * to SSL connect through the proxy -- and we don't know if we
    * will need to use SSL until we parse the url ...
    */
-  if (2 > sscanf(data->url, "%64[^\n:]://%256[^\n/]%" URL_MAX_LENGTH_TXT "[^\n]",
-		   proto, gname, path)) {
+  if((1 == sscanf(data->url, "file://%" URL_MAX_LENGTH_TXT "[^\n]",
+                  path))) {
+    /* we deal with file://<host>/<path> differently since it
+       supports no hostname other than "localhost" and "127.0.0.1",
+       which ist unique among the protocols specified in RFC 1738 */
+    if (strstr(path, "localhost/") || strstr(path, "127.0.0.1/"))
+      strcpy(path, &path[10]);		/* ... since coincidentally
+					   both host strings are of
+					   equal length */
+    /* otherwise, <host>/ is quietly ommitted */
+
+
+    /* that's it, no more fiddling with proxies, redirections,
+       or SSL for files, go directly to the file reading function */
+    result = file(data, path, &bytecount);
+    if(result)
+      return result;
+  
+    return URG_OK;
+  }
+  else if (2 > sscanf(data->url, "%64[^\n:]://%256[^\n/]%" URL_MAX_LENGTH_TXT "[^\n]",
+                 proto, gname, path)) {
+    
       
     /* badly formatted, let's try the browser-style _without_ 'http://' */
     if((1 > sscanf(data->url, "%256[^\n/]%" URL_MAX_LENGTH_TXT "[^\n]", gname,
@@ -665,12 +696,12 @@ static UrgError _urlget(struct UrlData *data)
     else if(strnequal(gname, "HTTPS", 5))
       strcpy(proto, "https");
 #endif /* USE_SSLEAY */
-#ifdef TRY_TELNET
     else if(strnequal(gname, "TELNET", 6))
       strcpy(proto, "telnet");
-#endif
     else if (strnequal(gname, "DICT", sizeof("DICT")-1))
       strcpy(proto, "DICT");
+    else if (strnequal(gname, "LDAP", sizeof("LDAP")-1))
+      strcpy(proto, "LDAP");
     else
       strcpy(proto, "http");
 
@@ -678,18 +709,18 @@ static UrgError _urlget(struct UrlData *data)
   }
 
 
-  if(data->conf & CONF_USERPWD) {
+  if((data->conf & CONF_USERPWD) && ! (data->conf & CONF_NETRC)) {
     if(':' != *data->userpwd) {
       if((1 <= sscanf(data->userpwd, "%127[^:]:%127s",
-                      ftpuser, ftppasswd))) {
+                      data->user, data->passwd))) {
         /* check for password, if no ask for one */
-        if( !ftppasswd[0] )
+        if( !data->passwd[0] )
         {
-          strncpy(ftppasswd, getpass("password: "), sizeof(ftppasswd));
+          strncpy(data->passwd, getpass("password: "), sizeof(data->passwd));
         }
       }
     }
-    if(!ftpuser[0]) {
+    if(!data->user[0]) {
       failf(data, "USER malformat: user name can't be zero length");
       return URG_MALFORMAT_USER;
     }
@@ -698,15 +729,15 @@ static UrgError _urlget(struct UrlData *data)
   if(data->conf & CONF_PROXYUSERPWD) {
     if(':' != *data->proxyuserpwd) {
       if((1 <= sscanf(data->proxyuserpwd, "%127[^:]:%127s",
-                      proxyuser, proxypasswd))) {
+                      data->proxyuser, data->proxypasswd))) {
         /* check for password, if no ask for one */
-        if( !proxypasswd[0] )
+        if( !data->proxypasswd[0] )
         {
-          strncpy(proxypasswd, getpass("proxy password: "), sizeof(proxypasswd));
+          strncpy(data->proxypasswd, getpass("proxy password: "), sizeof(data->proxypasswd));
         }
       }
     }
-    if(!proxyuser[0]) {
+    if(!data->proxyuser[0]) {
       failf(data, " Proxy USER malformat: user name can't be zero length");
       return URG_MALFORMAT_USER;
     }
@@ -741,10 +772,14 @@ static UrgError _urlget(struct UrlData *data)
 
       nope=no_proxy?strtok(no_proxy, ", "):NULL;
       while(nope) {
-	if(strequal(nope, name)) {
-	  /* no proxy for this host! */
-	  break;
-	}
+        if(strlen(nope) <= strlen(name)) {
+          char *checkn=
+            name + strlen(name) - strlen(nope);
+          if(strnequal(nope, checkn, strlen(nope))) {
+            /* no proxy for this host! */
+            break;
+          }
+        }
 	nope=strtok(NULL, ", ");
       }
       if(!nope) {
@@ -795,6 +830,8 @@ static UrgError _urlget(struct UrlData *data)
     if(data->freethis)
       free(data->freethis);
     data->freethis = reurl;
+
+    data->conf &= ~CONF_NOPROT; /* switch that one off again */
   }
 
   /* RESUME on a HTTP page is a tricky business. First, let's just check that
@@ -891,7 +928,6 @@ static UrgError _urlget(struct UrlData *data)
       }
     }
   }
-#ifdef TRY_TELNET
   else if(strequal(proto, "TELNET")) {
     /* telnet testing factory */
     data->conf |= CONF_TELNET;
@@ -899,29 +935,44 @@ static UrgError _urlget(struct UrlData *data)
       data->port = PORT_TELNET;
     data->remote_port = PORT_TELNET;
   }
-#endif
   else if (strequal(proto, "DICT")) {
     data->conf |= CONF_DICT;
     if(!data->port)
       data->port = PORT_DICT;
     data->remote_port = PORT_DICT;
   }
+  else if (strequal(proto, "LDAP")) {
+    data->conf |= CONF_LDAP;
+    if(!data->port)
+      data->port = PORT_LDAP;
+    data->remote_port = PORT_LDAP;
+  }
+  /* file:// is handled above */
+  /*  else if (strequal(proto, "FILE")) {
+    data->conf |= CONF_FILE;
+
+    result = file(data, path, &bytecount);
+    if(result)
+      return result;
+
+    return URG_OK;
+    }*/
   else {
     failf(data, "Unsupported protocol: %s", proto);
     return URG_UNSUPPORTED_PROTOCOL;
   }
 
   if(data->conf & CONF_NETRC) {
-    if(ParseNetrc(data->hostname, ftpuser, ftppasswd)) {
+    if(ParseNetrc(data->hostname, data->user, data->passwd)) {
       infof(data, "Couldn't find host %s in the .netrc file, using defaults",
             data->hostname);
     }
     /* weather we failed or not, we don't know which fields that were filled
        in anyway */
-    if(!ftpuser[0])
-      strcpy(ftpuser, "anonymous");
-    if(!ftppasswd[0])
-      strcpy(ftppasswd, "curl_by_Daniel.Stenberg@sth.frontec.se");
+    if(!data->user[0])
+      strcpy(data->user, CURL_DEFAULT_USER);
+    if(!data->passwd[0])
+      strcpy(data->passwd, CURL_DEFAULT_PASSWORD);
     if(data->conf & CONF_HTTP) {
       data->conf |= CONF_USERPWD;
     }
@@ -938,13 +989,13 @@ static UrgError _urlget(struct UrlData *data)
       return URG_URL_MALFORMAT_USER;
     }
     if((1 <= sscanf(name, "%127[^:]:%127[^@]",
-		    ftpuser, ftppasswd)) && (ptr=strchr(name, '@'))) {
+		    data->user, data->passwd)) && (ptr=strchr(name, '@'))) {
       name = ++ptr;
       data->conf |= CONF_USERPWD;
     }
     else {
-      strcpy(ftpuser, "anonymous");
-      strcpy(ftppasswd, "curl_by_Daniel.Stenberg@sth.frontec.se");
+      strcpy(data->user, CURL_DEFAULT_USER);
+      strcpy(data->passwd, CURL_DEFAULT_PASSWORD);
     }
   }
 
@@ -1001,6 +1052,7 @@ static UrgError _urlget(struct UrlData *data)
       return URG_COULDNT_RESOLVE_PROXY;
     }
   }
+  pgrsTime(data, TIMER_NAMELOOKUP);
 
   data->firstsocket = socket(AF_INET, SOCK_STREAM, 0);
 
@@ -1033,7 +1085,7 @@ static UrgError _urlget(struct UrlData *data)
 
   if(data->conf & CONF_PROXYUSERPWD) {
     char authorization[512];
-    sprintf(data->buffer, "%s:%s", proxyuser, proxypasswd);
+    sprintf(data->buffer, "%s:%s", data->proxyuser, data->proxypasswd);
     base64Encode(data->buffer, authorization);
 
     data->ptr_proxyuserpwd = maprintf("Proxy-authorization: Basic %s\015\012",
@@ -1097,6 +1149,7 @@ static UrgError _urlget(struct UrlData *data)
        return URG_SSL_CONNECT_ERROR;
      }
   }
+  pgrsTime(data, TIMER_CONNECT);
 
   now = tvnow(); /* time this *after* the connect is done */
   bytecount = 0;
@@ -1109,495 +1162,29 @@ static UrgError _urlget(struct UrlData *data)
   }
 
   if((data->conf&(CONF_FTP|CONF_PROXY)) == CONF_FTP) {
-    result = ftp(data, &bytecount, ftpuser, ftppasswd, ppath);
+    result = ftp(data, &bytecount, data->user, data->passwd, ppath);
     if(result)
       return result;
   }
-#ifdef TRY_TELNET
-  /***********************************************************************
-   *
-   * Telnet.
-   *
-   * - Require user+password.
-   *
-   * 1. Negotiation.
-   * 2. Read until login: prompt.
-   * 3. Send login.
-   * 4. Read until password: prompt.
-   * 5. Send password.
-   * 6. Send extra command (echo ---)
-   * 7. Wait for extra command output, when recieived we know we're in.
-   * 8. Run command; echo <boundary>
-   * 9. Eat everything until the first newline
-   * 10. Dump everything
-   * 11. If <boundary> is found, abort.
-   *
-   * Still very experimental.
-   *
-   **********************************************************************/
-   
-
   else if(data->conf & CONF_TELNET) {
-
-    typedef enum {
-      TELNET_CONNECT,
-      TELNET_LOGIN,
-      TELNET_PASSWD,
-      TELNET_INSIDE,
-      TELNET_DUMP
-    } TelnetState;
-
-#include <arpa/telnet.h>
-    unsigned char telopt[] =
-    {IAC,WONT,TELOPT_TTYPE,
-     IAC,DO, TELOPT_SGA,
-     IAC,WONT,TELOPT_XDISPLOC,
-     IAC,WONT,TELOPT_NAWS,
-     IAC,WONT,TELOPT_OLD_ENVIRON,
-     IAC,WONT,TELOPT_NEW_ENVIRON,
-     IAC,DONT,TELOPT_ECHO};
-    
-    TelnetState state=TELNET_CONNECT;
-
-    char *endstring;
-    int newlines=0;
-    int endlen;
-    int endmatch=0;
-
-    swrite(data->firstsocket,telopt,21);   /* kludge some telnet negotiation */
-
-    while (1) {
-      unsigned char cmd;
-      char *buffer;
-      char *ptr;
-
-      sread(data->firstsocket, data->buffer, 1);
-
-      if((unsigned char)data->buffer[0] > 250) {
-	cmd = data->buffer[0];
-	/* 251 - WILL
-	   252 - WONT
-	   253 - DO
-	   254 - DONT
-	   255 - IAC */
-	sread(data->firstsocket, data->buffer, 2);
-#if 0
-	fprintf(stderr, "%u %u %u\n",
-		(unsigned char)cmd,
-		(unsigned char)data->buffer[0],
-		(unsigned char)data->buffer[1]);
-#endif
-      }
-      else {
-	buffer = telnetin(data, data->buffer[0]);
-
-	if(state == TELNET_DUMP) {
-	  if(newlines) {
-	    if(!*buffer) {
-	      /* newline indication, decrease */
-	      newlines--;
-	    }
-	  }
-	  else {
-
-	    
-	    if(data->buffer[0] == endstring[endmatch]) {
-	      endmatch++; /* one (more) letter matches */
-
-	      if(endmatch == endlen) {
-		/* this is end of data */
-		free(endstring);
-		break;
-	      }
-	    }
-	    else {
-	      if(endmatch) {
-		fwrite(endstring, endmatch, 1, stdout);
-		endmatch=0;
-	      }
-	      printf("%c", data->buffer[0]);
-	    }
-	  }
-	}
-
-	/*	fprintf(stderr, "buffer: %s\n", buffer); */
-
-	if((state == TELNET_CONNECT) &&
-	   strequal("login: ", buffer)) {
-	  infof(data, "Got the login: prompt!\n");
-	  sendf(data->firstsocket, data, "%s\r\n", ftpuser); 
-	  state=TELNET_LOGIN; /* we have passed login */
-	}
-	else if((state == TELNET_LOGIN) &&
-		strequal("Password: ", buffer)) {
-	  infof(data, "Got the Password: prompt!\n");
-	  sendf(data->firstsocket, data, "%s\r\n", ftppasswd); 
-	  state=TELNET_PASSWD; /* we have passed password */
-
-	  sendf(data->firstsocket, data, "echo ---\r\n"); 
-	}
-	else if((state == TELNET_PASSWD) &&
-		strequal("---", buffer)) {
-	  infof(data, "We're logged in!\n");
-	  endstring = MakeFormBoundary();
-
-	  sendf(data->firstsocket, data, "%s;echo %s\r\n",
-		ppath+1, endstring);
-	  state=TELNET_INSIDE;
-	}
-	else if((state == TELNET_INSIDE) &&
-		(ptr = strstr(buffer, ppath+1)) ) {
-	  endlen = strlen(endstring);
-	  endmatch=0;
-
-	  state = TELNET_DUMP;
-	  newlines=1; /* swallow 2 newlines, then go */
-	}
-      }
-    }
+    result=telnet(data);
+    if(result)
+      return result;
+  }
+  else if (data->conf & CONF_LDAP) {
+    result = ldap(data, path, &bytecount);
+    if (result)
+      return result;
   }
-#endif
   else if (data->conf & CONF_DICT) {
-    int nth;
-    char *word;
-    char *database = NULL;
-    char *strategy = NULL;
-    char *nthdef = NULL; /* This is not part of the protocol, but required
-			    by RFC 2229 */
-    
-    if(data->conf & CONF_USERPWD) {
-      /* AUTH is missing */
-    }
-
-    if (strnequal(path, DICT_MATCH, sizeof(DICT_MATCH)-1) ||
-	strnequal(path, DICT_MATCH2, sizeof(DICT_MATCH2)-1) ||
-	strnequal(path, DICT_MATCH3, sizeof(DICT_MATCH3)-1)) {
-      
-      word = strchr(path, ':');
-      if (word) {
-	word++;
-	database = strchr(word, ':');
-	if (database) {
-	  *database++ = (char)0;
-	  strategy = strchr(database, ':');
-	  if (strategy) {
-	    *strategy++ = (char)0;
-	    nthdef = strchr(strategy, ':');
-	    if (nthdef) {
-	      *nthdef++ = (char)0;
-	    }
-	  }
-	}
-      }
-      
-      if ((word == NULL) || (*word == (char)0)) {
-	failf(data, "lookup word is missing\n");
-      }
-      if ((database == NULL) || (*database == (char)0)) {
-	database = "!";
-      }
-      if ((strategy == NULL) || (*strategy == (char)0)) {
-	strategy = ".";
-      }
-      if ((nthdef == NULL) || (*nthdef == (char)0)) {
-	nth = 0;
-      } else {
-	nth = atoi(nthdef);
-      }
-      
-      sendf(data->firstsocket, data,
-	    "CLIENT " LIBCURL_NAME " " LIBCURL_VERSION "\n"
-	    "MATCH "
-	    "%s "    /* database */
-	    "%s "    /* strategy */
-	    "%s\n"   /* word */
-	    "QUIT\n",
-	    
-	    database,
-	    strategy,
-	    word
-	    );
-    
-      result = Download(data, data->firstsocket, -1, FALSE, &bytecount);
-      
-      if(result)
-	return result;
-      
-    } else if (strnequal(path, DICT_DEFINE, sizeof(DICT_DEFINE)-1) ||
-	       strnequal(path, DICT_DEFINE2, sizeof(DICT_DEFINE2)-1) ||
-	       strnequal(path, DICT_DEFINE3, sizeof(DICT_DEFINE3)-1)) {
-      
-      word = strchr(path, ':');
-      if (word) {
-	word++;
-	database = strchr(word, ':');
-	if (database) {
-	  *database++ = (char)0;
-	  nthdef = strchr(database, ':');
-	  if (nthdef) {
-	    *nthdef++ = (char)0;
-	  }
-	}
-      }
-      
-      if ((word == NULL) || (*word == (char)0)) {
-	failf(data, "lookup word is missing\n");
-      }
-      if ((database == NULL) || (*database == (char)0)) {
-	database = "!";
-      }
-      if ((nthdef == NULL) || (*nthdef == (char)0)) {
-	nth = 0;
-      } else {
-	nth = atoi(nthdef);
-      }
-      
-      sendf(data->firstsocket, data,
-	    "CLIENT " LIBCURL_NAME " " LIBCURL_VERSION "\n"
-	    "DEFINE "
-	    "%s "     /* database */
-	    "%s\n"    /* word */
-	    "QUIT\n",
-
-	    database,
-	    word
-	    );
-    
-      result = Download(data, data->firstsocket, -1, FALSE, &bytecount);
-      
-      if(result)
-	return result;
-      
-    } else {
-      
-      ppath = strchr(path, '/');
-      if (ppath) {
-	int i;
-	
-	ppath++;
-	for (i = 0; (i < URL_MAX_LENGTH) && (ppath[i]); i++) {
-	  if (ppath[i] == ':')
-	    ppath[i] = ' ';
-	}
-	sendf(data->firstsocket, data,
-	      "CLIENT " LIBCURL_NAME " " LIBCURL_VERSION "\n"
-	      "%s\n"
-	      "QUIT\n",
-	      ppath);
-
-	result = Download(data, data->firstsocket, -1, FALSE, &bytecount);
-      
-	if(result)
-	  return result;
-	
-      }
-    }
-    
-    ProgressEnd(data);
+    result = dict(data, path, &bytecount);
+    if(result)
+      return result;
   }
   else {
-    /* Send the GET line to the HTTP server */
-
-    struct FormData *sendit=NULL;
-    int postsize=0;
-
-    if((data->conf&(CONF_HTTP|CONF_UPLOAD)) ==
-       (CONF_HTTP|CONF_UPLOAD)) {
-      /* enable PUT! */
-      data->conf |= CONF_PUT;
-    }
-
-    if(data->conf & CONF_USERPWD) {
-      char authorization[512];
-      sprintf(data->buffer, "%s:%s", ftpuser, ftppasswd);
-      base64Encode(data->buffer, authorization);
-      data->ptr_userpwd = maprintf( "Authorization: Basic %s\015\012",
-				    authorization);
-    }
-    if(data->conf & CONF_RANGE) {
-      data->ptr_rangeline = maprintf("Range: bytes=%s\015\012", data->range);
-    }
-    if(data->conf & CONF_REFERER) {
-      data->ptr_ref = maprintf("Referer: %s\015\012", data->referer);
-    }
-    if(data->cookie) {
-      data->ptr_cookie = maprintf("Cookie: %s\015\012", data->cookie);
-    }
-
-    if ((data->conf & CONF_PROXY) && (!(data->conf & CONF_HTTPS)))  {
-      /* The path sent to the proxy is in fact the entire URL */
-      strncpy(ppath, data->url, URL_MAX_LENGTH-1);
-    }
-    if(data->conf & CONF_HTTPPOST) {
-      /* we must build the whole darned post sequence first, so that we have
-	 a size of the whole shebang before we start to send it */
-      sendit = getFormData(data->httppost, &postsize);
-    }
-    do {
-      sendf(data->firstsocket, data,
-            "%s " /* GET/HEAD/POST/PUT */
-            "%s HTTP/1.0\r\n" /* path */
-            "%s" /* proxyuserpwd */
-            "%s" /* userpwd */
-            "%s" /* range */
-            "%s" /* user agent */
-            "%s" /* cookie */
-            "Host: %s\r\n" /* host */
-#ifdef MULTIDOC
-            "Connection: Keep-Alive\r\n"
-#endif
-            "Pragma: no-cache\r\n"
-            "Accept: image/gif, image/x-xbitmap, image/jpeg, image/pjpeg, */*\r\n"
-            "%s", /* referer */
-
-            data->conf&CONF_NOBODY?"HEAD":
-            (data->conf&(CONF_POST|CONF_HTTPPOST))?"POST":
-            (data->conf&CONF_PUT)?"PUT":
-            "GET",
-            ppath,
-            (data->conf&CONF_PROXYUSERPWD)?data->ptr_proxyuserpwd:"",
-            (data->conf&CONF_USERPWD)?data->ptr_userpwd:"",
-            (data->conf&CONF_RANGE)?data->ptr_rangeline:"",
-            (data->useragent && *data->useragent)?data->ptr_uagent:"",
-            (data->ptr_cookie?data->ptr_cookie:""), /* Cookie: <data> */
-            name, /* host */
-            (data->conf&CONF_REFERER)?data->ptr_ref:"" /* Referer: <data> <CRLF> */
-            );
-
-      if(data->cookies) {
-        struct Cookie *co = cookie_getlist(data->cookies,
-                                           name,
-                                           ppath,
-                                           data->conf&CONF_HTTPS?TRUE:FALSE);
-        /* now loop through all cookies that matched */
-        while(co) {
-          sendf(data->firstsocket, data,
-                "Cookie: %s=%s\r\n",
-                co->name, co->value);
-          co = co->next; /* next cookie please */
-        }
-        cookie_freelist(co); /* free the cookie list */
-      }
-
-      while(data->headers) {
-        sendf(data->firstsocket, data,
-              "%s\015\012",
-              data->headers->header);
-        data->headers = data->headers->next;
-      }
-
-      if(data->conf&(CONF_POST|CONF_HTTPPOST)) {
-        if(data->conf & CONF_POST) {
-          /* this is the simple x-www-form-urlencoded style */
-          sendf(data->firstsocket, data,
-                "Content-Length: %d\015\012"
-                "Content-Type: application/x-www-form-urlencoded\r\n\r\n"
-                "%s\015\012",
-                strlen(data->postfields),
-                data->postfields );
-        }
-        else {
-          struct Form form;
-          size_t (*storefread)(char *, size_t , size_t , FILE *);
-          FILE *in;
-          long conf;
-
-          if(FormInit(&form, sendit)) {
-            failf(data, "Internal HTTP POST error!\n");
-            return URG_HTTP_POST_ERROR;
-          }
-
-          storefread = data->fread; /* backup */
-          in = data->in; /* backup */
-          
-          data->fread =
-            (size_t (*)(char *, size_t, size_t, FILE *))
-            FormReader; /* set the read function to read from the
-                           generated form data */
-          data->in = (FILE *)&form;
-
-          sendf(data->firstsocket, data,
-                "Content-Length: %d\r\n",
-                postsize-2);
-	
-          conf = data->conf;
-          data->conf &= ~CONF_NOPROGRESS; /* enable progress meter */
-          ProgressInit(data, postsize);
-
-          result = Upload(data, data->firstsocket, &bytecount);
-
-          data->conf = conf; /* restore conf values for the download */
-          
-          if(result)
-            return result;
-	
-          data->fread = storefread; /* restore */
-          data->in = in; /* restore */
-#if 0
-          /* for hard debugging only */
-          while (1) {
-            sread(data->firstsocket, data->buffer, 1);
-            fprintf(stderr, "*%c", data->buffer[0]);
-          }
-#endif
-
-	sendf(data->firstsocket, data,
-	      "\r\n\r\n");
-        }
-      }
-      else if(data->conf&CONF_PUT) {
-        /* Let's PUT the data to the server! */
-        long conf;
-
-        if(data->infilesize>0) {
-          sendf(data->firstsocket, data,
-                "Content-Length: %d\r\n\r\n", /* file size */
-                data->infilesize );
-        }
-        else
-          sendf(data->firstsocket, data,
-                "\015\012");
-        
-        conf = data->conf;
-        data->conf &= ~CONF_NOPROGRESS; /* enable progress meter */
-
-        ProgressInit(data, data->infilesize);
-
-        result = Upload(data, data->firstsocket, &bytecount);
-        
-        data->conf = conf;
-
-        if(result)
-          return result;
-
-        bytecount=0;
-      }
-      else {
-        sendf(data->firstsocket, data, "\r\n");
-      }
-      /* HTTP GET/HEAD download: */
-      result = Download(data, data->firstsocket, -1, TRUE, &bytecount);
-
-      if(result)
-        return result;
-      
-      ProgressEnd(data);
-#ifdef MULTIDOC
-      if(data->moredoc) {
-        ppath = data->moredoc->path; /* new path to get */
-        fclose(data->out);
-        if(data->moredoc->store)
-          data->out = data->moredoc->store; /* new FILE * to write to */
-        else
-          data->out = stdout;
-        data->moredoc = data->moredoc->next; /* advance */
-      }
-      else
-        break;
-    } while (1);
-#else
-    } while (0); 
-#endif
+    result = http(data, ppath, name, &bytecount);
+    if(result)
+      return result;
   }
   if(bytecount) {
     double ittook = tvdiff (tvnow(), now);
