@@ -30,10 +30,10 @@
  *
  * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/lib/urldata.h,v $
- * $Revision: 1.15 $
- * $Date: 1999/09/13 22:32:08 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/lib/urldata.h,v $
+ * $Revision: 1.6 $
+ * $Date: 2000/03/01 22:06:57 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
@@ -42,7 +42,7 @@
 
 /* This file is for lib internal stuff */
 
-#include "../include/setup.h"
+#include "setup.h"
 
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 256
@@ -70,14 +70,24 @@
     
 #ifdef USE_SSLEAY
 /* SSLeay stuff usually in /usr/local/ssl/include */
-#include "rsa.h"       
+#ifdef USE_OPENSSL
+#include "openssl/rsa.h"
+#include "openssl/crypto.h"
+#include "openssl/x509.h"
+#include "openssl/pem.h"
+#include "openssl/ssl.h"
+#include "openssl/err.h"
+#else
+#include "rsa.h"
 #include "crypto.h"
 #include "x509.h"
 #include "pem.h"
 #include "ssl.h"
 #include "err.h"
 #endif
+#endif
 
+#include "timeval.h"
 
 /* Download buffer size, keep it fairly big for speed reasons */
 #define BUFSIZE (1024*50)
@@ -86,9 +96,35 @@
    of need. */
 #define HEADERSIZE 256
 
+struct Progress {
+  long lastshow; /* time() of the last displayed progress meter or NULL to
+                    force redraw at next call */
+  double size_dl;
+  double size_ul;
+  double downloaded;
+  double uploaded;
+
+  double current_speed; /* uses the currently fastest transfer */
+
+  int mode;  /* what kind of progress meter to display */
+  int width; /* screen width at download start */
+  int flags; /* see progress.h */
+  double timespent;
+  double dlspeed;
+  double ulspeed;
+
+  struct timeval start;
+  /* various data stored for possible later report */
+  struct timeval t_nslookup;
+  struct timeval t_connect;
+  struct timeval t_pretransfer;
+  int httpcode;
+};
+
 struct UrlData {
-  FILE *out;   /* the fetched file goes here */
-  FILE *in;    /* the uploaded file is read from here */
+  FILE *out;    /* the fetched file goes here */
+  FILE *in;     /* the uploaded file is read from here */
+  FILE *err;    /* the stderr writes goes here */
   FILE *writeheader; /* write the header to this is non-NULL */
   char *url;   /* what to get */
   char *freethis; /* if non-NULL, an allocated string for the URL */
@@ -125,6 +161,9 @@ struct UrlData {
   long timeout; /* in seconds, 0 means no timeout */
   long infilesize; /* size of file to upload, -1 means unknown */
 
+  long maxdownload; /* in bytes, the maximum amount of data to fetch, 0
+                       means unlimited */
+  
   /* fields only set and used within _urlget() */
   int firstsocket;     /* the main socket to use */
   int secondarysocket; /* for i.e ftp transfers */
@@ -145,9 +184,6 @@ struct UrlData {
   char *newurl; /* This can only be set if a Location: was in the
 		   document headers */
 
-#ifdef MULTIDOC
-  struct MoreDoc *moredoc;  /* linked list of more docs to get */
-#endif
   struct HttpHeader *headers; /* linked list of extra headers */
   struct HttpPost *httppost;  /* linked list of POST data */
 
@@ -163,16 +199,22 @@ struct UrlData {
   X509*    server_cert;
 #endif /* USE_SSLEAY */
   long crlf;
-  struct curl_slist *quote;
+  struct curl_slist *quote;     /* before the transfer */
+  struct curl_slist *postquote; /* after the transfer */
 
   TimeCond timecondition;
   time_t timevalue;
 
-  char *httprequest; /* http request to use */
+  char *customrequest; /* http/ftp request to use */
 
   char *headerbuff; /* allocated buffer to store headers in */
   int headersize;   /* size of the allocation */
 
+  char *writeinfo;  /* if non-NULL describes what to output on a successful
+                       completion */
+
+  struct Progress progress;
+
 #define MAX_CURL_USER_LENGTH 128
 #define MAX_CURL_PASSWORD_LENGTH 128
 
@@ -188,16 +230,9 @@ struct UrlData {
   char *ptr_rangeline; /* free later if not NULL! */
   char *ptr_ref; /* free later if not NULL! */
   char *ptr_cookie; /* free later if not NULL! */
+  char *ptr_host; /* free later if not NULL */
 };
 
-#ifdef USE_SSLEAY
-#define SSL_ID "(SSL enabled)"
-#define LIBCURL_SSL " SSL" /* with space to better append to other strings! */
-#else
-#define SSL_ID "(SSL disabled)"
-#define LIBCURL_SSL ""
-#endif
-
 #define LIBCURL_NAME "libcurl"
 #define LIBCURL_ID LIBCURL_NAME " " LIBCURL_VERSION " " SSL_ID
 
