@@ -24,14 +24,14 @@
  *
  * ------------------------------------------------------------
  * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@sth.frontec.se>
+ * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
  *
- * 	http://www.fts.frontec.se/~dast/curl/
+ * 	http://curl.haxx.nu
  *
- * $Source: /cvs/curl/curl/src/main.c,v $
- * $Revision: 1.10 $
- * $Date: 1999/04/19 22:23:43 $
- * $Author: dast $
+ * $Source: /cvsroot/curl/src/main.c,v $
+ * $Revision: 1.11 $
+ * $Date: 2000/03/19 19:55:02 $
+ * $Author: bagder $
  * $State: Exp $
  * $Locker:  $
  *
@@ -45,19 +45,46 @@
 #include <sys/stat.h>
 #include <ctype.h>
 
-#include "../include/setup.h"
-#include "../include/curl.h"
-#include "../include/mprintf.h"
+#include <curl/curl.h>
+#include <curl/mprintf.h>
+#include "../lib/getdate.h"
+#ifdef GLOBURL
+#include "urlglob.h"
+#define CURLseparator	"--_curl_--"
+#define MIMEseparator	"_curl_"
+#endif
+
+/* This is now designed to have its own local setup.h */
+#include "setup.h"
 
 #include "version.h"
 
-#ifdef WIN32
+#ifdef HAVE_IO_H /* typical win32 habit */
 #include <io.h>
-#else
+#endif
+
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#endif
+
+#ifndef HAVE_STRDUP
+/* Ultrix doesn't have strdup(), so make a quick clone: */
+char *strdup(char *str)
+{
+  int len;
+  char *newstr;
+
+  len = strlen(str);
+  newstr = (char *) malloc((len+1)*sizeof(char));
+  if (!newstr)
+    return (char *)NULL;
+
+  strcpy(newstr,str);
+
+  return newstr;
+
+}
+#endif 
 
 extern void hugehelp(void);
 
@@ -82,48 +109,54 @@ static void help(void)
        " -A/--user-agent <string> User-Agent to send to server (H)\n"
        " -b/--cookie <name=string/file> Cookie string or file to read cookies from (H)\n"
        " -B/--ftp-ascii     Use ASCII transfer (F)\n"
-       " -c/--continue      Resume a previous transfer where we left it (F)\n"
-       " -C/--continue-at <offset> Specify absolute resume offset (F)\n"
+       " -c/--continue      Resume a previous transfer where we left it\n"
+       " -C/--continue-at <offset> Specify absolute resume offset\n"
        " -d/--data          POST data (H)\n"
-       " -D/--dump-header <file> Write the HTTP headers to this file (H)\n"
+       " -D/--dump-header <file> Write the headers to this file\n"
        " -e/--referer       Referer page (H)\n"
        " -E/--cert <cert:passwd> Specifies your certificate file and password (HTTPS)\n"
        " -f/--fail          Fail silently (no output at all) on errors (H)\n"
        " -F/--form <name=content> Specify HTTP POST data (H)\n"
-#ifdef MULTIDOC
-       " -g/--get-too <path> Specifies an extra file path to get (H)\n"
-       " -G/--get-store <file> File name to store the extra file as (H)\n"
-#endif
+
        " -h/--help          This help text\n"
        " -H/--header <line> Custom header to pass to server. (H)\n"
        " -i/--include       Include the HTTP-header in the output (H)\n"
-       " -I/--head          Fetch the HTTP-header only (HEAD) (H)\n"
+       " -I/--head          Fetch document info only (HTTP HEAD/FTP SIZE)\n"
        " -K/--config        Specify which config file to read\n"
        " -l/--list-only     List only names of an FTP directory (F)\n"
        " -L/--location      Follow Location: hints (H)\n"
        " -m/--max-time <seconds> Maximum time allowed for the transfer\n"
        " -M/--manual        Display huge help text\n"
        " -n/--netrc         Read .netrc for user name and password\n"
+       " -N/--no-buffer     Disables the buffering of the output stream\n"
        " -o/--output <file> Write output to <file> instead of stdout\n"
        " -O/--remote-name   Write output to a file named as the remote file\n"
+#if 0
        " -p/--port <port>   Use port other than default for current protocol.\n"
+#endif
        " -P/--ftpport <address> Use PORT with address instead of PASV when ftping (F)\n"
        " -q                 When used as the first parameter disables .curlrc\n"
        " -Q/--quote <cmd>   Send QUOTE command to FTP before file transfer (F)\n"
-       " -r/--range <range> Retrieve a byte range from a HTTP/1.1 server (H)\n"
-       " -s/--silent        Silent mode. Don't show progress info\n"
+       " -r/--range <range> Retrieve a byte range from a HTTP/1.1 or FTP server\n"
+       " -s/--silent        Silent mode. Don't output anything\n"
+       " -S/--show-error    Show error. With -s, make curl show errors when they occur\n"
        " -t/--upload        Transfer/upload stdin to remote site\n"
        " -T/--upload-file <file> Transfer/upload <file> to remote site\n"
        " -u/--user <user:password> Specify user and password to use\n"
        " -U/--proxy-user <user:password> Specify Proxy authentication\n"
        " -v/--verbose       Makes the operation more talkative\n"
        " -V/--version       Outputs version number then quits\n"
+       " -w/--write-out [format] What to output after completion\n"
        " -x/--proxy <host>  Use proxy. (Default port is 1080)\n"
-       " -y/--speed-limit   Stop transfer if below speed-limit for 'speed-time' secs\n"
-       " -Y/--speed-time    Time needed to trig speed-limit abort. Defaults to 30\n"
+       " -X/--request <command> Specific request command to use\n"
+       " -y/--speed-time    Time needed to trig speed-limit abort. Defaults to 30\n"
+       " -Y/--speed-limit   Stop transfer if below speed-limit for 'speed-time' secs\n"
+       " -z/--time-cond <time> Includes a time condition to the server (H)\n"
        " -2/--sslv2         Force usage of SSLv2 (H)\n"
        " -3/--sslv3         Force usage of SSLv3 (H)\n"
-       "    --crlf          Convert LF to CRLF in upload. Useful for MVS (OS/390)\n",
+       " -#/--progress-bar  Display transfer progress as a progress bar\n"
+       "    --crlf          Convert LF to CRLF in upload. Useful for MVS (OS/390)\n"
+       "    --stderr <file> Where to redirect stderr. - means stdout.\n",
          curl_version()
          );
 }
@@ -162,15 +195,21 @@ struct Configurable {
   char *cert_passwd;
   bool crlf;
   char *cookiefile;
+  char *customrequest;
+  bool progressmode;
+  bool nobuffer;
 
-  struct slist *quote;
+  char *writeout; /* %-styled format string to output */
+
+  FILE *errors; /* if stderr redirect is requested */
+
+  struct curl_slist *quote;
+  struct curl_slist *postquote;
 
   long ssl_version;
+  TimeCond timecond;
+  time_t condtime;
 
-#ifdef MULTIDOC
-  struct MoreDoc *newdoc;
-  struct MoreDoc *last_newdoc;
-#endif
   struct HttpHeader *headers;
   struct HttpHeader *last_header;
 
@@ -180,6 +219,8 @@ struct Configurable {
 
 static int parseconfig(char *filename,
 		       struct Configurable *config);
+static char *my_get_line(FILE *fp);
+static char *my_get_token(const char *line);
 
 static void GetStr(char **string,
 		   char *value)
@@ -189,6 +230,38 @@ static void GetStr(char **string,
   *string = strdup(value);
 }
 
+static char *file2string(FILE *file)
+{
+  char buffer[256];
+  char *ptr;
+  char *string=NULL;
+  int len=0;
+  int stringlen;
+
+  if(file) {
+    while(fgets(buffer, sizeof(buffer), file)) {
+      ptr= strchr(buffer, '\r');
+      if(ptr)
+        *ptr=0;
+      ptr= strchr(buffer, '\n');
+      if(ptr)
+        *ptr=0;
+      stringlen=strlen(buffer);
+      if(string)
+        string = realloc(string, len+stringlen+1);
+      else
+        string = malloc(stringlen+1);
+
+      strcpy(string+len, buffer);
+
+      len+=stringlen;
+    }
+    return string;
+  }
+  else
+    return NULL; /* no string */
+}
+
 static int getparameter(char *flag, /* f or -long-flag */
 			char *nextarg, /* NULL if unset */
 			bool *usedarg, /* set to TRUE if the arg has been
@@ -200,6 +273,7 @@ static int getparameter(char *flag, /* f or -long-flag */
   int res;
   struct HttpHeader *head;
   int j;
+  time_t now;
   int hit=-1;
 
   /* single-letter,
@@ -208,6 +282,8 @@ static int getparameter(char *flag, /* f or -long-flag */
      */
   struct LongShort aliases[]= {
     {"9", "crlf",        FALSE},
+    {"8", "stderr",      TRUE},
+
     {"2", "sslv2",       FALSE},
     {"3", "sslv3",       FALSE},
     {"a", "append",      FALSE},
@@ -222,10 +298,7 @@ static int getparameter(char *flag, /* f or -long-flag */
     {"E", "cert",        TRUE},
     {"f", "fail",        FALSE},
     {"F", "form",        TRUE},
-#ifdef MULTIDOC
-    {"g", "get-too",     TRUE},
-    {"G", "get-store",   TRUE},
-#endif
+
     {"h", "help",        FALSE},
     {"H", "header",      TRUE},
     {"i", "include",     FALSE},
@@ -236,23 +309,32 @@ static int getparameter(char *flag, /* f or -long-flag */
     {"m", "max-time",    TRUE},
     {"M", "manual",      FALSE},
     {"n", "netrc",       FALSE},
+    {"N", "no-buffer",   FALSE},
     {"o", "output",      TRUE},
     {"O", "remote-name", FALSE},
+#if 0
     {"p", "port",        TRUE},
+#endif
     {"P", "ftpport",     TRUE},
     {"q", "disable",     FALSE},
     {"Q", "quote",       TRUE},
     {"r", "range",       TRUE},
     {"s", "silent",      FALSE},
+    {"S", "show-error",  FALSE},
     {"t", "upload",      FALSE},
     {"T", "upload-file", TRUE},
     {"u", "user",        TRUE},
     {"U", "proxy-user",  TRUE},
     {"v", "verbose",     FALSE},
     {"V", "version",     FALSE},
+    {"w", "write-out",   TRUE},
     {"x", "proxy",       TRUE},
-    {"y", "speed-time",  TRUE},
-    {"Y", "speed-limit", TRUE},
+    {"X", "request",     TRUE},
+    {"X", "http-request", TRUE}, /* OBSOLETE VERSION */
+    {"Y", "speed-limit",  TRUE},
+    {"y", "speed-time", TRUE},
+    {"z", "time-cond",   TRUE},
+    {"#", "progress-bar",FALSE},
   };
 
   if('-' == flag[0]) {
@@ -319,10 +401,53 @@ static int getparameter(char *flag, /* f or -long-flag */
       *usedarg = TRUE; /* mark it as used */
 
     switch(letter) {
+    case 'z': /* time condition coming up */
+      switch(*nextarg) {
+      case '+':
+        nextarg++;
+      default:
+        /* If-Modified-Since: (section 14.28 in RFC2068) */
+        config->timecond = TIMECOND_IFMODSINCE;
+        break;
+      case '-':
+        /* If-Unmodified-Since:  (section 14.24 in RFC2068) */
+        config->timecond = TIMECOND_IFUNMODSINCE;
+        nextarg++;
+        break;
+      case '=':
+        /* Last-Modified:  (section 14.29 in RFC2068) */
+        config->timecond = TIMECOND_LASTMOD;
+        nextarg++;
+        break;
+      }
+      now=time(NULL);
+      config->condtime=get_date(nextarg, &now);
+      if(-1 == config->condtime) {
+        /* now let's see if it is a file name to get the time from instead! */
+        struct stat statbuf;
+        if(-1 == stat(nextarg, &statbuf)) {
+          /* failed, remove time condition */
+          config->timecond = TIMECOND_NONE;
+        }
+        else {
+          /* pull the time out from the file */
+          config->condtime = statbuf.st_mtime;
+        }
+      }
+      break;
     case '9': /* there is no short letter for this */
       /* LF -> CRLF conversinon? */
       config->crlf = TRUE;
       break;
+    case '8': /* there is no short letter for this */
+      if(strcmp(nextarg, "-"))
+        config->errors = fopen(nextarg, "wt");
+      else
+        config->errors = stdout;
+      break;
+    case '#': /* added 19990617 larsa */
+      config->progressmode ^= CURL_PROGRESS_BAR;
+      break;
     case '2': 
       /* SSL version 2 */
       config->ssl_version = 2;
@@ -340,14 +465,16 @@ static int getparameter(char *flag, /* f or -long-flag */
       GetStr(&config->useragent, nextarg);
       break;
     case 'b': /* cookie string coming up: */
-      if(strchr(nextarg, '=')) {
+      if(nextarg[0] == '@') {
+        nextarg++;
+      }
+      else if(strchr(nextarg, '=')) {
         /* A cookie string must have a =-letter */
         GetStr(&config->cookie, nextarg);
+        break;
       }
-      else {
-        /* We have a cookie file to read from! */
-        GetStr(&config->cookiefile, nextarg);
-      }
+      /* We have a cookie file to read from! */
+      GetStr(&config->cookiefile, nextarg);
       break;
     case 'B':
       /* use type ASCII when transfering ftp files */
@@ -364,9 +491,24 @@ static int getparameter(char *flag, /* f or -long-flag */
       break;
     case 'd':
       /* postfield data */
-      GetStr(&config->postfields, nextarg);
-      /* printf("%s\n",postfields); fflush(stdout); */
-      config->conf |= CONF_POST;
+      if('@' == *nextarg) {
+        /* the data begins with a '@' letter, it means that a file name
+           or - (stdin) follows */
+        FILE *file;
+        nextarg++; /* pass the @ */
+        if(strequal("-", nextarg))
+          file = stdin;
+        else 
+          file = fopen(nextarg, "r");
+        config->postfields = file2string(file);
+        if(file && (file != stdin))
+          fclose(stdin);
+      }
+      else {
+        GetStr(&config->postfields, nextarg);
+      }
+      if(config->postfields)
+        config->conf |= CONF_POST;
       break;
     case 'D':
       /* dump-header to given file name */
@@ -401,45 +543,7 @@ static int getparameter(char *flag, /* f or -long-flag */
 	return URG_FAILED_INIT;    
       config->conf |= CONF_HTTPPOST; /* no toggle, OR! */
       break;
-#ifdef MULTIDOC
-    case 'g':
-      /* specify an additional document to get from the same server as the
-         main URL */
-      {
-        struct MoreDoc *newdoc = (struct MoreDoc *)malloc(sizeof(struct MoreDoc));
-        if(newdoc) {
-          newdoc->next = NULL;
-          newdoc->store = NULL; /* FILE * to store this file at */
-          newdoc->path = NULL; /* first zero this */
-          GetStr(&newdoc->path, nextarg); /* now get the path */
 
-          /* point on our new one */
-          if(config->last_newdoc)
-            config->last_newdoc->next = newdoc;
-          else {
-            config->newdoc = newdoc;
-          }
-          config->last_newdoc = newdoc;
-        }
-      }
-    break;
-    case 'G':
-      {
-        FILE *docfile;
-        if(!config->last_newdoc ||
-           config->last_newdoc->store) {
-          /* bad usage, just ignore */
-        }
-        else {
-          docfile = fopen(nextarg, "w");
-          if(docfile)
-            config->last_newdoc->store = docfile;
-          else
-            fprintf(stderr, "Couldn't open file %s\n", nextarg);
-        }
-      }
-    break;
-#endif
     case 'h': /* h for help */
       help();
       return URG_FAILED_INIT;
@@ -491,6 +595,10 @@ static int getparameter(char *flag, /* f or -long-flag */
 	 automatically enfore user+password with the request */
       config->conf ^= CONF_NETRC;
       break;
+    case 'N':
+      /* disable the output I/O buffering */
+      config->nobuffer ^= 1;
+      break;
     case 'o':
       /* output file */
       GetStr(&config->outfile, nextarg); /* write to this file */
@@ -508,6 +616,7 @@ static int getparameter(char *flag, /* f or -long-flag */
       config->conf |= CONF_FTPPORT;
       GetStr(&config->ftpport, nextarg);
       break;
+#if 0
     case 'p':
       /* specified port */
       fputs("You've used the -p option, it will be removed in a future version\n",
@@ -515,12 +624,20 @@ static int getparameter(char *flag, /* f or -long-flag */
       config->porttouse = atoi(nextarg);
       config->conf |= CONF_PORT; /* changed port */
       break;
+#endif
     case 'q': /* if used first, already taken care of, we do it like
 		 this so we don't cause an error! */
       break;
     case 'Q':
       /* QUOTE command to send to FTP server */
-      config->quote = curl_slist_append(config->quote, nextarg);
+      if(nextarg[0] == '-') {
+        /* prefixed with a dash makes it a POST TRANSFER one */
+        nextarg++;
+        config->postquote = curl_slist_append(config->postquote, nextarg);
+      }
+      else {
+        config->quote = curl_slist_append(config->quote, nextarg);
+      }
       break;
     case 'r':
       /* byte range requested */
@@ -528,9 +645,13 @@ static int getparameter(char *flag, /* f or -long-flag */
       config->conf |= CONF_RANGE;
       break;
     case 's':
-      /* don't show progress meter: */
+      /* don't show progress meter, don't show errors : */
       config->conf |= (CONF_MUTE|CONF_NOPROGRESS);
-      config->showerror=FALSE;
+      config->showerror ^= TRUE; /* toggle off */
+      break;
+    case 'S':
+      /* show errors */
+      config->showerror ^= TRUE; /* toggle on if used with -s */
       break;
     case 't':
       /* we are uploading */
@@ -557,6 +678,24 @@ static int getparameter(char *flag, /* f or -long-flag */
     case 'V':
       printf(CURL_ID "%s\n", curl_version());
       return URG_FAILED_INIT;
+    case 'w':
+      /* get the output string */
+      if('@' == *nextarg) {
+        /* the data begins with a '@' letter, it means that a file name
+           or - (stdin) follows */
+        FILE *file;
+        nextarg++; /* pass the @ */
+        if(strequal("-", nextarg))
+          file = stdin;
+        else 
+          file = fopen(nextarg, "r");
+        config->writeout = file2string(file);
+        if(file && (file != stdin))
+          fclose(stdin);
+      }
+      else 
+        GetStr(&config->writeout, nextarg);
+      break;
     case 'x':
       /* proxy */
       if(!*nextarg) {
@@ -567,14 +706,18 @@ static int getparameter(char *flag, /* f or -long-flag */
 	config->conf |= CONF_PROXY;
 	GetStr(&config->proxy, nextarg);
       }
-    break;
-    case 'Y':
+      break;
+    case 'X':
+      /* HTTP request */
+      GetStr(&config->customrequest, nextarg);
+      break;
+    case 'y':
       /* low speed time */
       config->low_speed_time = atoi(nextarg);
       if(!config->low_speed_limit)
 	config->low_speed_limit = 1;
       break;
-    case 'y':
+    case 'Y':
       /* low speed limit */
       config->low_speed_limit = atoi(nextarg);
       if(!config->low_speed_time)
@@ -625,90 +768,58 @@ static int parseconfig(char *filename,
   else
     file = stdin;
   
-  if(file) {
-    char *tok;
+  if(file)
+  {
+    char *line;
+    char *tok1;
     char *tok2;
-    while(fgets(configbuffer, sizeof(configbuffer), file)) {
+
+    while (NULL != (line = my_get_line(file)))
+    {
       /* lines with # in the fist column is a comment! */
-
-#if 0
-      fprintf(stderr, "%s", configbuffer);
-#endif
-      if('#' == configbuffer[0])
-	continue;
-      tok = configbuffer;
-
-      while(*tok && isspace((int)*tok))
-	tok++;
-/*      tok=strtok(configbuffer, " \t\n"); */
-#if 0
-      fprintf(stderr, "TOK: %s\n", tok);
-#endif
-      if('-' != tok[0]) {
-	char *nl;
-	if(config->url)
-	  free(config->url);
-	config->url = strdup(tok);
-	nl = strchr(config->url, '\n');
-	if(nl)
-	  *nl=0;
+      if ('#' == line[0])
+      {
+        free(line);
+        continue;
       }
-      while(('-' == tok[0])) {
-	/* this is a flag */
-	char *firsttok = strdup(tok);
-	char *nl;
 
-	/* remove newline from firsttok */
-	nl = strchr(firsttok, '\n');
-	if(nl)
-	  *nl=0;
-
-	/* pass the -flag */
-	tok2=tok;
-	while(*tok2 && !isspace((int)*tok2))
-	  tok2++;
-
-	/* pass the following white space */
-	while(*tok2 && isspace((int)*tok2))
-	  tok2++;
-	
-	while(!*tok2 &&
-	      fgets(configbuffer, sizeof(configbuffer), file)) {
-	  /* lines with # in the fist column is a comment! */
-#if 0
-	  fprintf(stderr, "%s", configbuffer);
-#endif
-	  if('#' == configbuffer[0])
-	    continue;
-	  tok2 = configbuffer;
-	  /*	    tok2=strtok(configbuffer, " \t\n"); */
-	  /* pass white space */
-	  while(*tok2 && isspace((int)*tok2))
-	    tok2++;
-	}
-	/* remove newline from tok2 */
-	nl = strchr(tok2, '\n');
-	if(nl)
-	  *nl=0;
-
-	res = getparameter(firsttok+1,
-			   *tok2?tok2:NULL,
-			   &usedarg,
-			   config);
-	free(firsttok);
-#if 0
-	fprintf(stderr, "TOK %s TOK2: %s RES: %d\n",
-		firsttok, tok2?tok2:"NULL", res);
-#endif
-	if(res)
-	  return res;
-	if(!usedarg) {
-	  /* tok2 is unused,  */
-	  tok = tok2;
-	}
-	else 
-	  break; /* we've used both our words */
+      if (NULL == (tok1 = my_get_token(line)))
+      {
+        free(line);
+        continue;
       }
+      if ('-' != tok1[0])
+      {
+        if (config->url)
+          free(config->url);
+        config->url = tok1;
+      }
+
+      while ((NULL != tok1) && ('-' == tok1[0]))
+      {
+        tok2 = my_get_token(NULL);
+        while (NULL == tok2)
+        {
+          free(line);
+          if (NULL == (line = my_get_line(file)))
+            break;
+          if ('#' == line[0])
+            continue;
+          tok2 = my_get_token(line);
+        }
+
+        res = getparameter(tok1 + 1, tok2, &usedarg, config);
+        free(tok1);
+        if (!usedarg)
+          tok1 = tok2;
+        else
+        {
+          free(tok2);
+          break;
+        }
+      }
+
+      free(line);
     }
     if(file != stdin)
       fclose(file);
@@ -716,25 +827,65 @@ static int parseconfig(char *filename,
   return URG_OK;
 }
 
+struct OutStruct {
+  char *filename;
+  FILE *stream;
+};
+
+/* having this global is a bit dirty, but hey, who said we weren't? ;-) */
+struct Configurable config;
+
+int my_fwrite(void *buffer, size_t size, size_t nmemb, FILE *stream)
+{
+  struct OutStruct *out=(struct OutStruct *)stream;
+  if(out && !out->stream) {
+    /* open file for writing */
+    out->stream=fopen(out->filename, "wb");
+    if(!out->stream)
+      return -1; /* failure */
+    if(config.nobuffer) {
+      /* disable output buffering */
+#ifdef HAVE_SETVBUF
+      setvbuf(out->stream, NULL, _IONBF, 0);
+#endif
+    }
+  }
+  return fwrite(buffer, size, nmemb, out->stream);
+}
 
 
 int main(int argc, char *argv[])
 {
   char errorbuffer[URLGET_ERROR_SIZE];
 
+  struct OutStruct outs;
+  struct OutStruct heads;
+
   char *url = NULL;
+#ifdef GLOBURL
+  URLGlob *urls;
+  int urlnum;
+  char *outfiles = NULL;
+  int separator = 0;
+#endif
   
-  FILE *outfd = stdout;
   FILE *infd = stdin;
   FILE *headerfilep = NULL;
   char *urlbuffer=NULL;
   int infilesize=-1; /* -1 means unknown */
   bool stillflags=TRUE;
 
-  int res;
+  int res=URG_OK;
   int i;
 
-  struct Configurable config;
+  outs.stream = stdout;
+#ifdef __EMX__
+  /* 20000318 mgs
+   * we call _fsetmode to fix the problem with fwrite converting newline
+   * characters (you get mangled text files, and corrupted binary files when
+   * you download to stdout and redirect it to a file). */
+  _fsetmode(stdout, "b");
+#endif
 
   memset(&config, 0, sizeof(struct Configurable));
   
@@ -743,8 +894,10 @@ int main(int argc, char *argv[])
                              "%s", curl_version());
   config.showerror=TRUE;
   config.conf=CONF_DEFAULT;
+#if 0
   config.crlf=FALSE;
   config.quote=NULL;
+#endif
 
   if(argc>1 &&
      (!strnequal("--", argv[1], 2) && (argv[1][0] == '-')) &&
@@ -818,6 +971,26 @@ int main(int argc, char *argv[])
     fprintf(stderr, "URL: %s PROXY: %s\n", url, config.proxy?config.proxy:"none");
 #endif
 
+#ifdef GLOBURL
+  urlnum = glob_url(&urls, url);	/* expand '{...}' and '[...]' expressions and return
+					   total number of URLs in pattern set */
+  outfiles = config.outfile;		/* save outfile pattern befor expansion */
+  if (!outfiles && !config.remotefile && urlnum > 1) {
+#ifdef CURL_SEPARATORS
+    /* multiple files extracted to stdout, insert separators! */
+    separator = 1;
+#endif
+#ifdef MIME_SEPARATORS
+    /* multiple files extracted to stdout, insert MIME separators! */
+    separator = 1;
+    printf("MIME-Version: 1.0\n");
+    printf("Content-Type: multipart/mixed; boundary=%s\n\n", MIMEseparator);
+#endif
+  }
+  for (i = 0; (url = next_url(urls)); ++i) {
+    if (outfiles)
+      config.outfile = strdup(outfiles);
+#endif
 
   if(config.outfile && config.infile) {
     helpf("you can't both upload and download!\n");
@@ -843,6 +1016,10 @@ int main(int argc, char *argv[])
         return URG_WRITE_ERROR;
       }
     }
+#ifdef GLOBURL
+    else	/* fill '#1' ... '#9' terms from URL pattern */
+      config.outfile = match_url(config.outfile, *urls);
+#endif
 
     if((0 == config.resume_from) && config.use_resume) {
       /* we're told to continue where we are now, then we get the size of the
@@ -856,11 +1033,17 @@ int main(int argc, char *argv[])
       /* else let offset remain 0 */
     }
 
-    /* open file for output: */
-    outfd=(FILE *) fopen(config.outfile, config.resume_from?"ab":"wb");
-    if (!outfd) {
-      helpf("Can't open '%s'!\n", config.outfile);
-      return URG_WRITE_ERROR;
+    if(config.resume_from) {
+      /* open file for output: */
+      outs.stream=(FILE *) fopen(config.outfile, config.resume_from?"ab":"wb");
+      if (!outs.stream) {
+        helpf("Can't open '%s'!\n", config.outfile);
+        return URG_WRITE_ERROR;
+      }
+    }
+    else {
+      outs.filename = config.outfile;
+      outs.stream = NULL; /* open when needed */
     }
   }
   if (config.infile) {
@@ -911,27 +1094,43 @@ int main(int argc, char *argv[])
     /* open file for output: */
     if(strcmp(config.headerfile,"-"))
     {
-      headerfilep=(FILE *) fopen(config.headerfile, "wb");
-      if (!headerfilep) {
-	helpf("Can't open '%s'!\n", config.headerfile);
-	return URG_WRITE_ERROR;
-      }
+      heads.filename = config.headerfile;
+      headerfilep=NULL;
     }
     else
       headerfilep=stdout;
+    heads.stream = headerfilep;
   }
 
-  /* This was previously done in urlget, but that was wrong place to do it */
-  if(isatty(fileno(outfd)))
-    /* we send the output to a tty, and therefor we switch off the progress
-       meter right away */
+  if(outs.stream && isatty(fileno(outs.stream)) &&
+     !(config.conf&(CONF_UPLOAD|CONF_HTTPPOST)))
+    /* we send the output to a tty and it isn't an upload operation, therefore
+       we switch off the progress meter */
     config.conf |= CONF_NOPROGRESS;
 
-  res = curl_urlget(URGTAG_FILE, outfd,  /* where to store */
+#ifdef GLOBURL
+  if (urlnum > 1) {
+    fprintf(stderr, "\n[%d/%d]: %s --> %s\n", i+1, urlnum, url, config.outfile ? config.outfile : "<stdout>");
+    if (separator) {
+#ifdef CURL_SEPARATORS
+      printf("%s%s\n", CURLseparator, url);
+#endif
+#ifdef MIME_SEPARATORS
+      printf("--%s\n", MIMEseparator);
+      printf("Content-ID: %s\n\n", url); 
+#endif
+    }
+  }
+#endif
+
+  if(!config.errors)
+    config.errors = stderr;
+
+  res = curl_urlget(URGTAG_FILE, (FILE *)&outs,  /* where to store */
+                    URGTAG_WRITEFUNCTION, my_fwrite, /* what call to write */
                     URGTAG_INFILE, infd, /* for uploads */
                     URGTAG_INFILESIZE, infilesize, /* size of uploaded file */
                     URGTAG_URL, url,     /* what to fetch */
-                    URGTAG_PORT, config.porttouse, /* from which port */
                     URGTAG_PROXY, config.proxy, /* proxy to use */
                     URGTAG_FLAGS, config.conf, /* flags */
                     URGTAG_USERPWD, config.userpwd, /* user + passwd */
@@ -953,21 +1152,32 @@ int main(int argc, char *argv[])
                     URGTAG_SSLCERTPASSWD, config.cert_passwd,
                     URGTAG_CRLF, config.crlf,
                     URGTAG_QUOTE, config.quote,
-                    URGTAG_WRITEHEADER, headerfilep,
-#ifdef MULTIDOC
-                    URGTAG_MOREDOCS, config.newdoc,
-#endif
+                    URGTAG_POSTQUOTE, config.postquote,
+                    URGTAG_WRITEHEADER, config.headerfile?&heads:NULL,
                     URGTAG_COOKIEFILE, config.cookiefile,
                     URGTAG_SSLVERSION, config.ssl_version,
+                    URGTAG_TIMECONDITION, config.timecond,
+                    URGTAG_TIMEVALUE, config.condtime,
+                    URGTAG_CUSTOMREQUEST, config.customrequest,
+                    URGTAG_STDERR, config.errors,
+                    URGTAG_PROGRESSMODE, config.progressmode,
+                    URGTAG_WRITEINFO, config.writeout,
                     URGTAG_DONE); /* always terminate the list of tags */
-
   if((res!=URG_OK) && config.showerror)
-    fprintf(stderr, "curl: %s\n", errorbuffer);
+    fprintf(config.errors, "curl: (%d) %s\n", res, errorbuffer);
+
+  if((config.errors != stderr) &&
+     (config.errors != stdout))
+    /* it wasn't directed to stdout or stderr so close the file! */
+    fclose(config.errors);
+
+  if(config.headerfile && !headerfilep && heads.stream)
+    fclose(heads.stream);
 
   if(urlbuffer)
     free(urlbuffer);
-  if (config.outfile)
-    fclose(outfd);
+  if (config.outfile && outs.stream)
+    fclose(outs.stream);
   if (config.infile)
     fclose(infd);
   if(headerfilep)
@@ -976,8 +1186,80 @@ int main(int argc, char *argv[])
   if(config.url)
     free(config.url);
 
+#ifdef GLOBURL
+  if(url)
+    free(url);
+  if(config.outfile && !config.remotefile)
+    free(config.outfile);
+  }
+#ifdef MIME_SEPARATORS
+  if (separator)
+    printf("--%s--\n", MIMEseparator);
+#endif
+#endif
+
   curl_slist_free_all(config.quote); /* the checks for config.quote == NULL */
 
   return(res);
 }
 
+static char *my_get_line(FILE *fp)
+{
+   char buf[4096];
+   char *nl = NULL;
+   char *retval = NULL;
+
+   do
+   {
+      if (NULL == fgets(buf, sizeof(buf), fp))
+         break;
+      if (NULL == retval)
+         retval = strdup(buf);
+      else
+      {
+         if (NULL == (retval = realloc(retval,
+                                       strlen(retval) + strlen(buf) + 1)))
+            break;
+         strcat(retval, buf);
+      }
+   }
+   while (NULL == (nl = strchr(retval, '\n')));
+
+   if (NULL != nl)
+     *nl = '\0';
+
+   return retval;
+}
+
+static char *my_get_token(const char *line)
+{
+  static const char *save = NULL;
+  const char *first = NULL;
+  const char *last = NULL;
+  char *retval = NULL;
+  size_t size;
+
+  if (NULL == line)
+    line = save;
+  if (NULL == line)
+    return NULL;
+
+  while (('\0' != *line) && (isspace(*line)))
+    line++;
+  first = line;
+  while (('\0' != *line) && (!isspace(*line)))
+    line++;
+  save = line;
+  while ('\0' != *line)
+    line++;
+  last = line;
+
+  size = last - first;
+  if (0 == size)
+    return NULL;
+  if (NULL == (retval = malloc(size + 1)))
+    return NULL;
+  memcpy(retval, first, size);
+  retval[size] = '\0';
+  return retval;
+}
